"""Hooks are dependencies that can be used across components.

A example use case would be a settings hook to persist user preferences
or a shopping cart hook to manage the user's shopping cart. This unlocks
easy access to shared state across components without the need for
bubbling state up and down the component tree.

However hooks may not always be the first solution, as they can obscure
functionality through a layer of abstraction.
"""

from functools import cache
from typing import TYPE_CHECKING, cast

from qtpy.QtCore import QObject
from qtpy.QtCore import Signal as QtSignal  # pyright: ignore[reportPrivateImportUsage]
from typing_extensions import Self

if TYPE_CHECKING:
    from material_ui._component import Signal


class Hook(QObject):
    """Base class for hooks."""

    on_change = cast("Signal", QtSignal())
    """Signal emitted when the hook's internal state has changed."""

    @classmethod
    @cache
    def get(cls) -> Self:
        """Get the singleton instance."""
        return cls()



"""Design tokens for md.comp.date-picker."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

docked_container_color = _define_token("md.sys.color.surface-container-high")
docked_container_elevation = _define_token("md.sys.elevation.level3")
docked_container_height = _define_token(456)
docked_container_shape = _define_token("md.sys.shape.corner.large")
docked_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
docked_container_width = _define_token(360)
docked_date_container_height = _define_token(48)
docked_date_container_shape = _define_token("md.sys.shape.corner.full")
docked_date_container_width = _define_token(48)
docked_date_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
docked_date_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
docked_date_label_text_font = _define_token("md.sys.typescale.body-large.font")
docked_date_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
docked_date_label_text_size = _define_token("md.sys.typescale.body-large.size")
docked_date_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
docked_date_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
docked_date_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
docked_date_selected_container_color = _define_token("md.sys.color.primary")
docked_date_selected_focus_state_layer_color = _define_token("md.sys.color.on-primary")
docked_date_selected_hover_state_layer_color = _define_token("md.sys.color.on-primary")
docked_date_selected_label_text_color = _define_token("md.sys.color.on-primary")
docked_date_selected_pressed_state_layer_color = _define_token("md.sys.color.on-primary")
docked_date_state_layer_height = _define_token(40)
docked_date_state_layer_shape = _define_token("md.sys.shape.corner.full")
docked_date_state_layer_width = _define_token(40)
docked_date_today_container_outline_color = _define_token("md.sys.color.primary")
docked_date_today_container_outline_width = _define_token(1)
docked_date_today_focus_state_layer_color = _define_token("md.sys.color.primary")
docked_date_today_hover_state_layer_color = _define_token("md.sys.color.primary")
docked_date_today_label_text_color = _define_token("md.sys.color.primary")
docked_date_today_pressed_state_layer_color = _define_token("md.sys.color.primary")
docked_date_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_date_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_date_unselected_label_text_color = _define_token("md.sys.color.on-surface")
docked_date_unselected_outside_month_label_text_color = _define_token("md.sys.color.outline-variant")
docked_date_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_header_height = _define_token(64)
docked_menu_button_container_height = _define_token(40)
docked_menu_button_container_shape = _define_token("md.sys.shape.corner.full")
docked_menu_button_disabled_icon_color = _define_token("md.sys.color.on-surface")
docked_menu_button_disabled_icon_opacity = _define_token(0.38)
docked_menu_button_disabled_label_text_color = _define_token("md.sys.color.on-surface")
docked_menu_button_disabled_label_text_opacity = _define_token(0.38)
docked_menu_button_focus_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_focus_label_text_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
docked_menu_button_hover_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
docked_menu_button_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_icon_size = _define_token(18)
docked_menu_button_label_text_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_label_text_font = _define_token("md.sys.typescale.label-large.font")
docked_menu_button_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
docked_menu_button_label_text_size = _define_token("md.sys.typescale.label-large.size")
docked_menu_button_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
docked_menu_button_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
docked_menu_button_pressed_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_pressed_label_text_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_button_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
docked_menu_list_item_container_height = _define_token(48)
docked_menu_list_item_focus_label_text_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_focus_state_layer_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
docked_menu_list_item_hover_label_text_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_hover_state_layer_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
docked_menu_list_item_label_text_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_label_text_font = _define_token("md.sys.typescale.body-large.font")
docked_menu_list_item_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
docked_menu_list_item_label_text_size = _define_token("md.sys.typescale.body-large.size")
docked_menu_list_item_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
docked_menu_list_item_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
docked_menu_list_item_pressed_label_text_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
docked_menu_list_item_selected_container_color = _define_token("md.sys.color.surface-variant")
docked_menu_list_item_selected_focus_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_list_item_selected_hover_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_menu_list_item_selected_leading_icon_color = _define_token("md.sys.color.on-surface")
docked_menu_list_item_selected_leading_icon_size = _define_token(24)
docked_menu_list_item_selected_pressed_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
docked_weekdays_label_text_color = _define_token("md.sys.color.on-surface")
docked_weekdays_label_text_font = _define_token("md.sys.typescale.body-large.font")
docked_weekdays_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
docked_weekdays_label_text_size = _define_token("md.sys.typescale.body-large.size")
docked_weekdays_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
docked_weekdays_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
modal_container_color = _define_token("md.sys.color.surface-container-high")
modal_container_elevation = _define_token("md.sys.elevation.level3")
modal_container_height = _define_token(568)
modal_container_shape = _define_token("md.sys.shape.corner.extra-large")
modal_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
modal_container_width = _define_token(360)
modal_date_container_height = _define_token(40)
modal_date_container_shape = _define_token("md.sys.shape.corner.full")
modal_date_container_width = _define_token(40)
modal_date_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
modal_date_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
modal_date_label_text_font = _define_token("md.sys.typescale.body-large.font")
modal_date_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
modal_date_label_text_size = _define_token("md.sys.typescale.body-large.size")
modal_date_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
modal_date_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
modal_date_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
modal_date_selected_container_color = _define_token("md.sys.color.primary")
modal_date_selected_focus_state_layer_color = _define_token("md.sys.color.on-primary")
modal_date_selected_hover_state_layer_color = _define_token("md.sys.color.on-primary")
modal_date_selected_label_text_color = _define_token("md.sys.color.on-primary")
modal_date_selected_pressed_state_layer_color = _define_token("md.sys.color.on-primary")
modal_date_state_layer_height = _define_token(40)
modal_date_state_layer_shape = _define_token("md.sys.shape.corner.full")
modal_date_state_layer_width = _define_token(40)
modal_date_today_container_outline_color = _define_token("md.sys.color.primary")
modal_date_today_container_outline_width = _define_token(1)
modal_date_today_focus_state_layer_color = _define_token("md.sys.color.primary")
modal_date_today_hover_state_layer_color = _define_token("md.sys.color.primary")
modal_date_today_label_text_color = _define_token("md.sys.color.primary")
modal_date_today_pressed_state_layer_color = _define_token("md.sys.color.primary")
modal_date_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
modal_date_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
modal_date_unselected_label_text_color = _define_token("md.sys.color.on-surface")
modal_date_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
modal_header_container_height = _define_token(120)
modal_header_container_width = _define_token(360)
modal_header_headline_color = _define_token("md.sys.color.on-surface-variant")
modal_header_headline_font = _define_token("md.sys.typescale.headline-large.font")
modal_header_headline_line_height = _define_token("md.sys.typescale.headline-large.line-height")
modal_header_headline_size = _define_token("md.sys.typescale.headline-large.size")
modal_header_headline_tracking = _define_token("md.sys.typescale.headline-large.tracking")
modal_header_headline_weight = _define_token("md.sys.typescale.headline-large.weight")
modal_header_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
modal_header_supporting_text_font = _define_token("md.sys.typescale.label-large.font")
modal_header_supporting_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
modal_header_supporting_text_size = _define_token("md.sys.typescale.label-large.size")
modal_header_supporting_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
modal_header_supporting_text_weight = _define_token("md.sys.typescale.label-large.weight")
modal_range_selection_active_indicator_container_color = _define_token("md.sys.color.secondary-container")
modal_range_selection_active_indicator_container_height = _define_token(40)
modal_range_selection_active_indicator_container_shape = _define_token("md.sys.shape.corner.full")
modal_range_selection_container_elevation = _define_token("md.sys.elevation.level0")
modal_range_selection_container_shape = _define_token("md.sys.shape.corner.none")
modal_range_selection_date_in_range_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
modal_range_selection_date_in_range_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
modal_range_selection_date_in_range_focus_state_layer_opcaity = _define_token("md.sys.state.focus.state-layer-opacity")
modal_range_selection_date_in_range_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
modal_range_selection_date_in_range_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
modal_range_selection_date_in_range_hover_state_layer_opcaity = _define_token("md.sys.state.hover.state-layer-opacity")
modal_range_selection_date_in_range_label_text_color = _define_token("md.sys.color.on-secondary-container")
modal_range_selection_date_in_range_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
modal_range_selection_date_in_range_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
modal_range_selection_date_in_range_pressed_state_layer_opcaity = _define_token("md.sys.state.pressed.state-layer-opacity")
modal_range_selection_header_container_height = _define_token(128)
modal_range_selection_header_headline_font = _define_token("md.sys.typescale.title-large.font")
modal_range_selection_header_headline_line_height = _define_token("md.sys.typescale.title-large.line-height")
modal_range_selection_header_headline_size = _define_token("md.sys.typescale.title-large.size")
modal_range_selection_header_headline_tracking = _define_token("md.sys.typescale.title-large.tracking")
modal_range_selection_header_headline_weight = _define_token("md.sys.typescale.title-large.weight")
modal_range_selection_month_subhead_color = _define_token("md.sys.color.on-surface-variant")
modal_range_selection_month_subhead_font = _define_token("md.sys.typescale.title-small.font")
modal_range_selection_month_subhead_line_height = _define_token("md.sys.typescale.title-small.line-height")
modal_range_selection_month_subhead_size = _define_token("md.sys.typescale.title-small.size")
modal_range_selection_month_subhead_tracking = _define_token("md.sys.typescale.title-small.tracking")
modal_range_selection_month_subhead_weight = _define_token("md.sys.typescale.title-small.weight")
modal_weekdays_label_text_color = _define_token("md.sys.color.on-surface")
modal_weekdays_label_text_font = _define_token("md.sys.typescale.body-large.font")
modal_weekdays_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
modal_weekdays_label_text_size = _define_token("md.sys.typescale.body-large.size")
modal_weekdays_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
modal_weekdays_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
modal_year_selection_year_container_height = _define_token(36)
modal_year_selection_year_container_width = _define_token(72)
modal_year_selection_year_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
modal_year_selection_year_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
modal_year_selection_year_label_text_font = _define_token("md.sys.typescale.body-large.font")
modal_year_selection_year_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
modal_year_selection_year_label_text_size = _define_token("md.sys.typescale.body-large.size")
modal_year_selection_year_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
modal_year_selection_year_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
modal_year_selection_year_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
modal_year_selection_year_selected_container_color = _define_token("md.sys.color.primary")
modal_year_selection_year_selected_focus_state_layer_color = _define_token("md.sys.color.on-primary")
modal_year_selection_year_selected_hover_state_layer_color = _define_token("md.sys.color.on-primary")
modal_year_selection_year_selected_label_text_color = _define_token("md.sys.color.on-primary")
modal_year_selection_year_selected_pressed_state_layer_color = _define_token("md.sys.color.on-primary")
modal_year_selection_year_state_layer_height = _define_token(36)
modal_year_selection_year_state_layer_shape = _define_token("md.sys.shape.corner.full")
modal_year_selection_year_state_layer_width = _define_token(72)
modal_year_selection_year_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
modal_year_selection_year_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
modal_year_selection_year_unselected_label_text_color = _define_token("md.sys.color.on-surface-variant")
modal_year_selection_year_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")



"""Design tokens for md.comp.sheet."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

bottom_docked_container_color = _define_token("md.sys.color.surface-container-low")
bottom_docked_container_shape = _define_token("md.sys.shape.corner.extra-large.top")
bottom_docked_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
bottom_docked_drag_handle_color = _define_token("md.sys.color.on-surface-variant")
bottom_docked_drag_handle_height = _define_token(4)
bottom_docked_drag_handle_opacity = _define_token(0.4)
bottom_docked_drag_handle_width = _define_token(32)
bottom_docked_minimized_container_shape = _define_token("md.sys.shape.corner.none")
bottom_docked_modal_container_elevation = _define_token("md.sys.elevation.level1")
bottom_docked_standard_container_elevation = _define_token("md.sys.elevation.level1")
bottom_focus_indicator_color = _define_token("md.sys.color.secondary")
bottom_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
bottom_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
side_detached_container_shape = _define_token("md.sys.shape.corner.large")
side_docked_action_focus_label_text_color = _define_token("md.sys.color.primary")
side_docked_action_focus_state_layer_color = _define_token("md.sys.color.primary")
side_docked_action_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
side_docked_action_hover_label_text_color = _define_token("md.sys.color.primary")
side_docked_action_hover_state_layer_color = _define_token("md.sys.color.primary")
side_docked_action_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
side_docked_action_pressed_label_text_color = _define_token("md.sys.color.primary")
side_docked_action_pressed_state_layer_color = _define_token("md.sys.color.primary")
side_docked_action_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
side_docked_container_color = _define_token("md.sys.color.surface")
side_docked_container_height = _define_token(1.0)
side_docked_container_shape = _define_token("md.sys.shape.corner.none")
side_docked_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
side_docked_container_width = _define_token(256)
side_docked_divider_color = _define_token("md.sys.color.outline")
side_docked_focus_indicator_color = _define_token("md.sys.color.secondary")
side_docked_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
side_docked_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
side_docked_headline_color = _define_token("md.sys.color.on-surface-variant")
side_docked_headline_font = _define_token("md.sys.typescale.title-large.font")
side_docked_headline_line_height = _define_token("md.sys.typescale.title-large.line-height")
side_docked_headline_size = _define_token("md.sys.typescale.title-large.size")
side_docked_headline_tracking = _define_token("md.sys.typescale.title-large.tracking")
side_docked_headline_weight = _define_token("md.sys.typescale.title-large.weight")
side_docked_modal_container_color = _define_token("md.sys.color.surface-container-low")
side_docked_modal_container_elevation = _define_token("md.sys.elevation.level1")
side_docked_modal_container_shape = _define_token("md.sys.shape.corner.large.start")
side_docked_standard_container_color = _define_token("md.sys.color.surface")
side_docked_standard_container_elevation = _define_token("md.sys.elevation.level0")



from pytestqt.qtbot import QtBot

from material_ui.progress_indicators import LinearProgress

# Taking a screenshot will cause the widget to be painted.


def test_LinearProgress_determinate_screenshot(qtbot: QtBot):
    progress_indicator = LinearProgress()
    progress_indicator.value = 0.5
    qtbot.addWidget(progress_indicator)
    assert qtbot.screenshot(progress_indicator) is not None


def test_LinearProgress_indeterminate_screenshot(qtbot: QtBot):
    progress_indicator = LinearProgress()
    progress_indicator.indeterminate = True
    qtbot.addWidget(progress_indicator)
    assert qtbot.screenshot(progress_indicator) is not None



"""Outlined text field component."""

from qtpy.QtCore import QMargins, QPoint

from material_ui._component import Component, effect
from material_ui.shape import Shape
from material_ui.text_fields._base_text_field import BaseTextField
from material_ui.tokens import md_comp_outlined_text_field as tokens
from material_ui.tokens import md_sys_color

_LINE_EDIT_POS = QPoint(16, 18)


class OutlinedTextField(BaseTextField):
    """Outlined text field component."""

    _FLOATING_LABEL_POS = QPoint(12, 0)
    _RESTING_LABEL_POS = QPoint(16, 24)

    def __init__(self) -> None:
        super().__init__()
        self._container = Shape()
        self._container.corner_shape = tokens.container_shape
        self._container.opacity = 0.0
        self._container.sx = {
            **self._container.sx,
            "border-width": tokens.outline_width,
            "border-style": "solid",
        }
        self._container.setParent(self)

        self._focus_outline = Shape()
        self._focus_outline.corner_shape = tokens.container_shape
        self._focus_outline.visible = self.focused
        self._focus_outline.opacity = 0.0
        self._focus_outline.sx = {
            **self._focus_outline.sx,
            "border-width": tokens.focus_outline_width,
            "border-style": "solid",
            "border-color": tokens.focus_outline_color,
        }
        self._focus_outline.setParent(self)

        self._resting_label.setParent(self)
        self._resting_label.font_family = tokens.label_text_font
        self._resting_label.font_size = tokens.label_text_size
        self._resting_label.font_weight = tokens.label_text_weight

        self._floating_label.setParent(self)
        self._floating_label.font_family = tokens.label_text_font
        self._floating_label.font_size = tokens.label_text_populated_size
        self._floating_label.font_weight = tokens.label_text_weight
        self._floating_label.sx = {
            **self._floating_label.sx,
            # TODO: split outline into 3 parts to make it all transparent
            "background-color": md_sys_color.background,
            "margin": "0 4px",
        }

        line_edit_wrapper = Component()
        line_edit_wrapper.sx = {
            "color": tokens.input_text_color,
            "font-family": tokens.input_text_font,
            "font-size": tokens.input_text_size,
        }
        line_edit_wrapper.setParent(self._container)
        line_edit_wrapper.move(_LINE_EDIT_POS)
        self._line_edit.setParent(line_edit_wrapper)

    @effect(Component.hovered)
    def _apply_container_border(self) -> None:
        color = tokens.hover_outline_color if self.hovered else tokens.outline_color
        self._container.sx = {**self._container.sx, "border-color": color}

    @effect(Component.size)
    def _resize_elements(self) -> None:
        self._container.resize(self.size().shrunkBy(QMargins(0, self._TOP_SPACE, 0, 0)))
        self._container.move(QPoint(0, self._TOP_SPACE))
        self._focus_outline.resize(self._container.size())
        self._focus_outline.move(QPoint(0, self._TOP_SPACE))

    @effect(Component.focused)
    def _apply_label_color(self) -> None:
        color = (
            tokens.focus_label_text_color if self.focused else tokens.label_text_color
        )
        self._floating_label.sx = {**self._floating_label.sx, "color": color}
        self._resting_label.sx = {**self._resting_label.sx, "color": color}



"""Design tokens for md.sys.typescale."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

body_large_font = _define_token("md.ref.typeface.plain")
body_large_line_height = _define_token(24)
body_large_size = _define_token(16)
body_large_tracking = _define_token(0.5)
body_large_weight = _define_token("md.ref.typeface.weight-regular")
body_medium_font = _define_token("md.ref.typeface.plain")
body_medium_line_height = _define_token(20)
body_medium_size = _define_token(14)
body_medium_tracking = _define_token(0.25)
body_medium_weight = _define_token("md.ref.typeface.weight-regular")
body_small_font = _define_token("md.ref.typeface.plain")
body_small_line_height = _define_token(16)
body_small_size = _define_token(12)
body_small_tracking = _define_token(0.4)
body_small_weight = _define_token("md.ref.typeface.weight-regular")
display_large_font = _define_token("md.ref.typeface.brand")
display_large_line_height = _define_token(64)
display_large_size = _define_token(57)
display_large_tracking = _define_token(-0.25)
display_large_weight = _define_token("md.ref.typeface.weight-regular")
display_medium_font = _define_token("md.ref.typeface.brand")
display_medium_line_height = _define_token(52)
display_medium_size = _define_token(45)
display_medium_tracking = _define_token(0)
display_medium_weight = _define_token("md.ref.typeface.weight-regular")
headline_large_font = _define_token("md.ref.typeface.brand")
headline_large_line_height = _define_token(40)
headline_large_size = _define_token(32)
headline_large_tracking = _define_token(0)
headline_large_weight = _define_token("md.ref.typeface.weight-regular")
headline_medium_font = _define_token("md.ref.typeface.brand")
headline_medium_line_height = _define_token(36)
headline_medium_size = _define_token(28)
headline_medium_tracking = _define_token(0)
headline_medium_weight = _define_token("md.ref.typeface.weight-regular")
headline_small_font = _define_token("md.ref.typeface.brand")
headline_small_line_height = _define_token(32)
headline_small_size = _define_token(24)
headline_small_tracking = _define_token(0)
headline_small_weight = _define_token("md.ref.typeface.weight-regular")
label_large_font = _define_token("md.ref.typeface.plain")
label_large_line_height = _define_token(20)
label_large_size = _define_token(14)
label_large_tracking = _define_token(0.1)
label_large_weight = _define_token("md.ref.typeface.weight-medium")
label_large_weight_prominent = _define_token("md.ref.typeface.weight-bold")
label_medium_font = _define_token("md.ref.typeface.plain")
label_medium_line_height = _define_token(16)
label_medium_size = _define_token(12)
label_medium_tracking = _define_token(0.5)
label_medium_weight = _define_token("md.ref.typeface.weight-medium")
label_medium_weight_prominent = _define_token("md.ref.typeface.weight-bold")
label_small_font = _define_token("md.ref.typeface.plain")
label_small_line_height = _define_token(16)
label_small_size = _define_token(11)
label_small_tracking = _define_token(0.5)
label_small_weight = _define_token("md.ref.typeface.weight-medium")
title_large_font = _define_token("md.ref.typeface.brand")
title_large_line_height = _define_token(28)
title_large_size = _define_token(22)
title_large_tracking = _define_token(0)
title_large_weight = _define_token("md.ref.typeface.weight-regular")
title_medium_font = _define_token("md.ref.typeface.plain")
title_medium_line_height = _define_token(24)
title_medium_size = _define_token(16)
title_medium_tracking = _define_token(0.15)
title_medium_weight = _define_token("md.ref.typeface.weight-medium")
title_small_font = _define_token("md.ref.typeface.plain")
title_small_line_height = _define_token(20)
title_small_size = _define_token(14)
title_small_tracking = _define_token(0.1)
title_small_weight = _define_token("md.ref.typeface.weight-medium")



"""Common imports for Material UI components."""

from material_ui.shape import Shape
from material_ui.switch import Switch
from material_ui import buttons
from material_ui.typography import Typography
from material_ui._component import Component, use_state, effect

__all__ = [
    "Component",
    "Shape",
    "Switch",
    "Typography",
    "use_state",
    "effect",
    "buttons",
]



"""Menu component.

A popup menu that opens at a specific location and displays a list of
selectable items.
"""

from typing import cast

from PySide6.QtGui import QMouseEvent
from qtpy.QtCore import QEasingCurve, QMargins, QPoint, Qt, QTimer

from material_ui._component import Component, effect, use_state
from material_ui._lab import DropShadow
from material_ui.icon import Icon
from material_ui.layout_basics import Row, Stack
from material_ui.ripple import Ripple
from material_ui.shape import Shape
from material_ui.theming.theme_hook import ThemeHook
from material_ui.tokens import md_comp_menu as tokens
from material_ui.tokens._utils import resolve_token, resolve_token_or_value
from material_ui.typography import Typography

_CONTAINER_DROP_SHADOW_SPACE = 10
"""Extra space around the menu container to accommodate the drop shadow."""

_DROP_SHADOW_MARGIN = QMargins(
    _CONTAINER_DROP_SHADOW_SPACE,
    _CONTAINER_DROP_SHADOW_SPACE,
    _CONTAINER_DROP_SHADOW_SPACE,
    _CONTAINER_DROP_SHADOW_SPACE,
)
_DIVIDER_MARGINS = QMargins(0, 8, 0, 8)
_CONTAINER_WIDTH_MIN = 112
_CONTAINER_WIDTH_MAX = 280
_LEFT_RIGHT_PADDING = 12
_GAP_BETWEEN_ELEMENTS_IN_ITEM = 12


class Menu(Component):
    """A popup menu that displays a list of selectable items."""

    def __init__(self) -> None:
        super().__init__()
        self.setWindowFlags(
            Qt.WindowType.Popup  # automatically closed on click outside
            | Qt.WindowType.NoDropShadowWindowHint  # use custom drop shadow
            | Qt.WindowType.FramelessWindowHint,  # prevent border
        )
        self.setAttribute(
            Qt.WidgetAttribute.WA_TranslucentBackground,
        )
        self.setMinimumWidth(_CONTAINER_WIDTH_MIN)
        self.setMaximumWidth(_CONTAINER_WIDTH_MAX)

        container = Shape()
        container.color = tokens.container_color
        container.corner_shape = tokens.container_shape

        drop_shadow = DropShadow()
        drop_shadow.shadow_color = tokens.container_shadow_color
        drop_shadow.elevation = tokens.container_elevation
        drop_shadow.setParent(container)
        container.setGraphicsEffect(drop_shadow)

        self._stack = Stack(margins=_DIVIDER_MARGINS)
        container.overlay_widget(self._stack)

        self.overlay_widget(container, margins=_DROP_SHADOW_MARGIN)

    def open(self, anchor_widget: Component) -> None:
        """Open the menu anchored to a specific widget.

        Args:
            anchor_widget: The widget to anchor the menu to.
        """
        pos = anchor_widget.mapToGlobal(QPoint(0, anchor_widget.height()))
        pos -= QPoint(0, _CONTAINER_DROP_SHADOW_SPACE)
        self.move(pos)
        self.show()

    def close_menu(self) -> None:
        """Close the menu."""
        self.destroy()

    @effect(Component.children)
    def _layout_menu_items(self) -> None:
        items = self.findChildren(
            MenuItem,
            options=Qt.FindChildOption.FindDirectChildrenOnly,
        )
        for item in items:
            item.clicked.connect(self._on_click_menu_item)
            self._stack.add_widget(item)

    def _on_click_menu_item(self) -> None:
        """Close menu when clicked something."""
        # Put a delay so the ripple can be enjoyed.
        QTimer.singleShot(50, self.close_menu)


class MenuItem(Component):
    """A single menu item."""

    text = use_state("")
    """Text displayed in the menu item."""

    leading_icon = use_state(cast("Icon | None", None))
    """Icon to go at the start of the item."""

    _state_layer_opacity = use_state(
        0.0,
        transition=70,
        easing=QEasingCurve.Type.InOutCubic,
    )

    def __init__(self) -> None:
        super().__init__()
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.setFixedHeight(resolve_token(tokens.list_item_container_height))

        row = Row()
        row.alignment = Qt.AlignmentFlag.AlignLeading | Qt.AlignmentFlag.AlignVCenter
        row.margins = QMargins(_LEFT_RIGHT_PADDING, 0, _LEFT_RIGHT_PADDING, 0)
        row.gap = _GAP_BETWEEN_ELEMENTS_IN_ITEM

        self._leading_icon_wrapper = Component()
        row.add_widget(self._leading_icon_wrapper)

        self._label = Typography()
        self._label.text = self.text
        self._label.font_family = tokens.list_item_label_text_font
        self._label.font_size = tokens.list_item_label_text_size
        self._label.font_weight = tokens.list_item_label_text_weight
        self._label.alignment = (
            Qt.AlignmentFlag.AlignLeading | Qt.AlignmentFlag.AlignVCenter
        )
        row.add_widget(self._label)

        self._state_layer = Shape()
        self._state_layer.setParent(self)
        self._state_layer.opacity = self._state_layer_opacity

        self._ripple = Ripple()
        self._ripple.color = tokens.list_item_pressed_state_layer_color
        self._ripple.clip_half_rounded = False
        self._state_layer.overlay_widget(self._ripple)

        self.overlay_widget(row)

    @effect(Component.size)
    def _apply_state_label_size(self) -> None:
        self._state_layer.resize(self.size())

    @effect(Component.hovered, Component.pressed)
    def _apply_state_layer_color(self) -> None:
        self._state_layer.color = (
            tokens.list_item_pressed_state_layer_color
            if self.pressed
            else tokens.list_item_hover_state_layer_color
        )
        self._state_layer_opacity = resolve_token_or_value(
            tokens.list_item_pressed_state_layer_opacity
            if self.pressed
            else tokens.list_item_hover_state_layer_opacity
            if self.hovered
            else 0.0,
        )

    def mousePressEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        """Override to handle ripple."""
        if event.button() == Qt.MouseButton.LeftButton:
            self._ripple.ripple_origin = event.position()
        return super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        """Override to handle ripple."""
        if event.button() == Qt.MouseButton.LeftButton:
            self._ripple.ripple_origin = None
        return super().mouseReleaseEvent(event)

    @effect(leading_icon)
    def _place_leading_icon(self) -> None:
        """Place the leading icon in the menu item."""
        # Delete previous icon if exists.
        if prev_icon := self._leading_icon_wrapper.findChild(Icon):
            if prev_icon is self.leading_icon:
                return  # Nothing to do.
            prev_icon.setParent(None)
        if self.leading_icon is None:
            # No icon needed.
            return
        # Show new icon.
        self._leading_icon_wrapper.overlay_widget(self.leading_icon)

    @effect(leading_icon, ThemeHook)
    def _apply_leading_icon_properties(self) -> None:
        icon = self.leading_icon
        if icon is None:
            return
        icon.font_size = tokens.list_item_with_leading_icon_leading_icon_size
        icon.color = tokens.list_item_with_leading_icon_leading_icon_color



import pytest
from pytest_mock import MockerFixture
from pytestqt.qtbot import QtBot

from material_ui._component import Component, EffectDependencyError, effect, use_state
from material_ui.hook import Hook


def test_Component_state_bind_on_assignment(qtbot: QtBot):
    class C(Component):
        a = use_state("")

    c1 = C()
    c1.a = "hello"
    qtbot.add_widget(c1)
    c2 = C()
    c2.a = "hey"
    qtbot.add_widget(c2)
    assert c1.a == "hello"
    assert c2.a == "hey"

    c2.a = c1.a
    assert c1.a == "hello"
    assert c2.a == "hello"

    c1.a = "hi"
    assert c1.a == "hi"
    assert c2.a == "hi"


def test_Component_effect_called_initially_and_on_change(
    qtbot: QtBot,
    mocker: MockerFixture,
):
    stub = mocker.stub()

    class C(Component):
        a = use_state("hello")

        @effect(a)
        def my_effect(self) -> None:
            stub(self.a)

    c = C()
    qtbot.add_widget(c)
    # Wait for the effect to be called after constructor.
    qtbot.wait_callback(timeout=0, raising=False).wait()

    # Check initial state call.
    stub.assert_called_once_with("hello")

    # New value assigned - effect should be called again.
    c.a = "hi"
    assert stub.call_count == 2
    stub.assert_called_with("hi")


def test_Component_effect_invalid_dependency_literal():
    with pytest.raises(EffectDependencyError):

        class C(Component):  # pyright: ignore[reportUnusedClass]
            @effect("hi")
            def my_effect(self) -> None:
                pass


def test_Component_effect_invalid_dependency_static():
    with pytest.raises(EffectDependencyError):

        class C(Component):  # pyright: ignore[reportUnusedClass]
            f = "hi"

            @effect(f)
            def my_effect(self) -> None:
                pass


def test_Component_effect_hook_dependency(qtbot: QtBot, mocker: MockerFixture):
    stub = mocker.stub()

    class MyHook(Hook):
        pass

    class MyComponent(Component):
        @effect(MyHook)
        def my_effect(self) -> None:
            stub()

    component = MyComponent()
    qtbot.add_widget(component)
    qtbot.wait(1)  # Let the effect be called after constructor.
    assert stub.call_count == 1

    MyHook.get().on_change.emit()
    assert stub.call_count == 2


def test_Component_effect_children_dependency(qtbot: QtBot, mocker: MockerFixture):
    stub = mocker.stub()

    class TestComponent(Component):
        @effect(Component.children)
        def my_effect(self) -> None:
            stub()

    parent = TestComponent()
    child = Component()
    qtbot.add_widget(parent)
    qtbot.wait(1)  # Let the effect be called after constructor.
    assert stub.call_count == 1

    child.setParent(parent)

    assert stub.call_count == 2



"""Material Icons (symbols)."""

from typing import Literal, cast

from qtpy.QtGui import QColor, QFont
from qtpy.QtWidgets import QLabel

from material_ui._component import Component, effect, use_state
from material_ui._font_utils import install_default_fonts
from material_ui.tokens import md_sys_color
from material_ui.tokens._utils import DesignToken, resolve_token_or_value

IconStyle = Literal[
    "outlined",
    "rounded",
    "sharp",
]


class Icon(Component):
    """Material icon component.

    Check here for available icons: https://fonts.google.com/icons
    """

    icon_name = use_state("star")
    """Icon name to display. See: https://fonts.google.com/icons."""

    icon_style = use_state(cast("IconStyle", "outlined"))
    """Icon style. Either 'outlined', 'rounded', or 'sharp'."""

    font_size = use_state(cast("DesignToken | int", 24))
    """Font size of icon."""

    color = use_state(cast("DesignToken | QColor", md_sys_color.on_surface))
    """Color of the icon."""

    weight = use_state(400)
    """Main control of thickness. 400 is default, 100 is thin, 700 is thick."""

    filled = use_state(False)
    """Whether the icon should be filled."""

    grade = use_state(0)
    """Thickness. 0 is default, -25 is thin, 200 is thick."""

    use_optical_size = use_state(True)
    """Whether to set the optical size to match the font size."""

    def __init__(self) -> None:
        super().__init__()

        # Ensure fonts are installed (blocking!).
        install_default_fonts()

        self._label = QLabel()
        self.overlay_widget(self._label, center=True)

    @effect(color)
    def _apply_color(self) -> None:
        self.sx = {**self.sx, "color": resolve_token_or_value(self.color)}

    @effect(icon_name)
    def _apply_icon_name(self) -> None:
        self._label.setText(self.icon_name)

    @effect(icon_style, filled, font_size, weight, grade, use_optical_size)
    def _apply_font(self) -> None:
        font_size = resolve_token_or_value(self.font_size)
        font = QFont("Material Symbols " + self.icon_style.title())
        font.setPixelSize(font_size)
        font.setVariableAxis(QFont.Tag("FILL"), 1 if self.filled else 0)
        font.setVariableAxis(QFont.Tag("wght"), self.weight)
        font.setVariableAxis(QFont.Tag("GRAD"), self.grade)
        optical_size = font_size if self.use_optical_size else 24
        font.setVariableAxis(QFont.Tag("opsz"), optical_size)
        self._label.setFont(font)



import pytest
import httpx
from pytest_mock import MockerFixture
from material_ui._font_utils import _get_cache_path_for_url, _download_font

_URL = "https://example.com/font.ttf"
_VALID_RESPONSE = httpx.Response(status_code=200, content=b"data")
_INVALID_RESPONSE = httpx.Response(status_code=500)


@pytest.fixture(autouse=True, scope="function")
def clear_cached_font() -> None:
    """Delete the cached font file before each test."""
    _get_cache_path_for_url(_URL).unlink(missing_ok=True)


@pytest.fixture
def client() -> httpx.AsyncClient:
    """Fixture to create a HTTP client for testing."""
    return httpx.AsyncClient()


@pytest.mark.asyncio
async def test__download_font_valid_response(
    mocker: MockerFixture, client: httpx.AsyncClient
) -> None:
    mocker.patch.object(client, "get", return_value=_VALID_RESPONSE)
    result = await _download_font(client, _URL)
    assert result is not None


@pytest.mark.asyncio
async def test__download_font_invalid_response(
    mocker: MockerFixture, client: httpx.AsyncClient
) -> None:
    mocker.patch.object(client, "get", return_value=_INVALID_RESPONSE)
    result = await _download_font(client, _URL)
    assert result is None


@pytest.mark.asyncio
async def test__download_font_no_refetch_if_cached(
    mocker: MockerFixture, client: httpx.AsyncClient
) -> None:
    mock_get = mocker.patch.object(client, "get", return_value=_VALID_RESPONSE)

    result1 = await _download_font(client, _URL)
    assert result1 is not None
    assert mock_get.call_count == 1

    # Already fetched, don't refetch
    result2 = await _download_font(client, _URL)
    assert result1 == result2
    assert mock_get.call_count == 1

    # With nocache, it should refetch
    result3 = await _download_font(client, _URL, no_cache=True)
    assert result1 == result3
    assert mock_get.call_count == 2


def test__get_cache_path_for_url_returns_valid_str() -> None:
    assert len(str(_get_cache_path_for_url(_URL))) > 10


def test__get_cache_path_for_url_returns_same_str() -> None:
    assert _get_cache_path_for_url(_URL) == _get_cache_path_for_url(_URL)
    assert _get_cache_path_for_url(_URL) != _get_cache_path_for_url(_URL + "2")



"""Sample of fixed and indeterminate, circular and linear progress indicators."""

from material_ui._component import Component
from material_ui.layout_basics import Row, Stack
from material_ui.progress_indicators.circular_progress import CircularProgress
from material_ui.progress_indicators.linear_progress import LinearProgress
from material_ui.tokens import md_sys_color
from qtpy.QtCore import QMargins
from qtpy.QtWidgets import QApplication


class ProgressIndicatorsSample(Component):
    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        row = Row()
        row.gap = 20

        circular = CircularProgress()
        circular.value = 0.75
        row.add_widget(circular)

        circular_indeterminate = CircularProgress()
        circular_indeterminate.indeterminate = True
        row.add_widget(circular_indeterminate)

        stack = Stack()
        stack.setFixedWidth(100)

        linear = LinearProgress()
        linear.value = 0.75
        stack.add_widget(linear)

        linear_indeterminate = LinearProgress()
        linear_indeterminate.indeterminate = True
        stack.add_widget(linear_indeterminate)

        row.add_widget(stack)

        self.overlay_widget(row, margins=QMargins(40, 30, 40, 30))


def main() -> None:
    app = QApplication()
    window = ProgressIndicatorsSample()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Internal widgets common functionality and helpers for Qt Widgets."""

import inspect
from collections.abc import Callable
from dataclasses import dataclass
from functools import partial
from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast, get_args, overload

from qtpy.QtCore import (
    Property,
    QChildEvent,  # pyright: ignore  # noqa: PGH003
    QEasingCurve,
    QEvent,
    QMargins,
    QObject,
    QPropertyAnimation,
    QSize,
    Qt,
    QTimer,
)
from qtpy.QtCore import (
    Signal as QtSignal,  # pyright: ignore  # noqa: PGH003
)
from qtpy.QtGui import QEnterEvent, QFocusEvent, QMouseEvent, QResizeEvent
from qtpy.QtWidgets import QGraphicsOpacityEffect, QVBoxLayout, QWidget
from typing_extensions import TypeIs, TypeVarTuple, Unpack

from material_ui._utils import StyleDict, convert_sx_to_qss, undefined
from material_ui.hook import Hook
from material_ui.theming.theme_hook import ThemeHook
from material_ui.tokens._utils import DesignToken, resolve_token_or_value

_Ts = TypeVarTuple("_Ts", default=Unpack[tuple[()]])


# TODO: should this be a protocol?
class Signal(Generic[Unpack[_Ts]]):
    """Type safe Qt signal wrapper type.

    This is to be used for type hinting only on class variables for
    classes deriving from `Component`. The actual value will be
    converted to a `QtCore.Signal` object in the class initialization.
    """

    def connect(self, slot: Callable[[Unpack[_Ts]], None]) -> None: ...
    def disconnect(self, slot: Callable[[Unpack[_Ts]], None]) -> None: ...
    def emit(self, *args: Unpack[_Ts]) -> None: ...


_T = TypeVar("_T")


@dataclass
class _TransitionConfig:
    """Transition properties for a state variable."""

    duration_ms: int
    easing: QEasingCurve.Type


class State(QObject, Generic[_T]):
    """Type safe property wrapper object.

    Creates a Qt property with a getter and setter and changed signal.

    Also used for one-way data binding and defined dependencies between
    variables.
    """

    changed: Signal[_T] = QtSignal("QVariant")  # type: ignore[assignment]

    def __init__(self, default_value: _T, name: str) -> None:
        super().__init__()
        self.setObjectName(name)
        self._value = default_value
        self._default_value = default_value
        self.transition: _TransitionConfig | None = None
        self._active_animation: QPropertyAnimation | None = None
        self._is_bound = False

    def get_value(self) -> _T:
        """Get the value of the variable."""
        return self._value

    def set_value(self, value: _T, *, _from_binding: bool = False) -> None:
        """Set the value of the variable.

        If the value changed, the changed signal is emitted.

        If the state is bound to another state, this function has no
        effect.

        Args:
            value: New value to use.
            _from_binding: Internal - whether the value was set from
                the state it is bound to.
        """
        if self._is_bound and not _from_binding:
            return
        if self._value != value:
            self._value = value
            self.changed.emit(value)

    def animate_to(
        self,
        value: _T,
        duration_ms: int,
        easing: QEasingCurve.Type,
    ) -> None:
        """Transition from current value to a new value.

        Args:
            value: The value to animate to.
            duration_ms: The duration of the animation in milliseconds.
            easing: The easing curve to use for the animation.
        """
        self._clear_active_animation()
        animation = QPropertyAnimation()
        animation.setParent(self)
        animation.setTargetObject(self)
        animation.setPropertyName(self._QT_PROPERTY_NAME.encode())
        animation.setDuration(duration_ms)
        animation.setEasingCurve(easing)
        animation.setStartValue(self._value)
        animation.setEndValue(value)
        animation.start()
        animation.finished.connect(self._clear_active_animation)
        self._active_animation = animation

    def _clear_active_animation(self) -> None:
        if self._active_animation:
            self._active_animation.setParent(None)
            self._active_animation.deleteLater()
            self._active_animation = None

    def set_transition(self, config: _TransitionConfig) -> None:
        """Automatically animate to the new value when set."""
        self.transition = config

    def bind(self, other: "State[_T]") -> None:
        """Bind this variable to another variable."""
        other.changed.connect(partial(self.set_value, _from_binding=True))
        self.set_value(other.get_value())  # Set initial state.
        self._is_bound = True
        # TODO: track object deletion

    def __repr__(self) -> str:
        parent_name = type(self.parent()).__name__ if self.parent() else "no-parent"
        return (
            f"<State '{self.objectName()}' of component '{parent_name}' "
            f"(current value: {str(self._value)[:20]})>"
        )

    _qt_property = Property("QVariant", get_value, set_value, None, "")
    """This is used by Qt to drive the animation."""

    _QT_PROPERTY_NAME = "_qt_property"
    """Name of the Qt property."""


def _find_signal_annotations(attrs: dict[str, Any]) -> dict[str, int]:
    """Find all signal annotations in the class attributes.

    Args:
        attrs: Class attributes dictionary.

    Returns:
        Dictionary of signal name and number of arguments.
    """
    ret_val: dict[str, int] = {}
    for key, value in attrs.get("__annotations__", {}).items():
        # Value is a Signal type hint, so need to get the actual type
        # out of it. This is needed for signals with no arguments, since
        # it's invalid syntax to write `Signal[]` with no arguments to
        # the square brackets. However this means the value is the
        # actual type not a _GenericAlias, so use it as the default arg.
        underlying_value = getattr(value, "__origin__", value)
        if underlying_value and issubclass(underlying_value, Signal):
            num_args = len(get_args(value))
            ret_val[key] = num_args
    return ret_val


class _ComponentMeta(type(QObject)):  # type: ignore[misc]
    """Meta class for all widgets."""

    def __new__(cls, name: str, bases: Any, attrs: Any) -> type:
        # Convert Signal annotations to actual Qt Signal objects.
        # Use QVariant to avoid runtime type checking by Qt. Can't
        # remember exact examples but it may fail for certain types.
        attrs.update(
            {
                key: QtSignal(*["QVariant"] * num_args)
                for key, num_args in _find_signal_annotations(attrs).items()
            },
        )
        return super().__new__(cls, name, bases, attrs)


@dataclass
class _StateMarker:
    """Marker to hold the default value on class variable."""

    name: str
    default_value: Any
    transition: int | None = None
    easing: QEasingCurve.Type = QEasingCurve.Type.Linear


def use_state(
    default_value: _T,
    transition: int | None = None,
    easing: QEasingCurve.Type = QEasingCurve.Type.Linear,
) -> _T:
    """Declare a state variable.

    This is intended to be used as a class variable. The default value
    will be used by all component instances, referring to the same
    value.

    Example:
        class MyComponent(Component):
            my_state = use_state("hello")

    Args:
        default_value: The default value of the state variable. This
            also sets the type of the variable, so it may be beneficial
            to use `cast` to show the full type (eg for optional values
            with initial state of None).
        transition: Optional transition duration, in milliseconds.
        easing: Optional easing curve to use for the transition.

    Returns:
        A marker object that will be replaced with the actual state once
        the object is constructed. The type annotation is intentionally
        incorrect to aid type annotations on the class variables.
        However it's only valid to use it in object instances, not as a
        static variable.
    """
    # This is the wrong type but assert it so that IDEs give completion
    # based on the expected return type.
    return _StateMarker(
        name="<unset>",
        default_value=default_value,
        transition=transition,
        easing=easing,
    )  # type: ignore[return-value]


def _find_state_markers(obj: object) -> list[_StateMarker]:
    """Find instances of use_state in the class.

    Args:
        obj: The object to search.

    Returns:
        List of state markers.
    """
    ret_val: list[_StateMarker] = []
    for name in dir(obj):
        value = getattr(obj, name)
        if isinstance(value, _StateMarker):
            value.name = name  # Set the name now we have access to it.
            ret_val.append(value)
    return ret_val


_EffectDependency = _StateMarker | type[Hook]
"""Effect dependency type."""


@dataclass
class _EffectMarker:
    """Marker to hold the dependencies of an effect."""

    name: str
    dependencies: list[_EffectDependency]


_EFFECT_MARKER_KEY = "__effect_marker__"


EffectFn = Callable[[Any], None]


class EffectDependencyError(RuntimeError):
    """Raised when the effect dependencies are invalid."""


def effect(*dependencies: Any) -> Callable[[EffectFn], EffectFn]:
    """Decorator to mark a method as an effect.

    The function will be called when the dependencies change, and also
    on the initial state.

    Args:
        dependencies: List of dependencies for the effect. These must be
            class variables marked with `use_state`.

    Returns:
        Decorated method.
    """
    # Special handling for Qt built in properties.
    # TODO: shadow these with actual states?
    dependencies_list = [
        x
        if isinstance(x, _StateMarker)
        else _StateMarker(name="_size", default_value=QSize())
        if x is QWidget.size
        else _StateMarker(name="_children", default_value=[])
        if x is QObject.children
        else x
        for x in dependencies
    ]

    # Validate dependency types.
    for dependency in dependencies_list:
        if not _is_valid_effect_dependency(dependency):
            msg = f"Invalid dependency for effect: {dependency} ({type(dependency)})"
            raise EffectDependencyError(msg)

    def decorated(func: EffectFn) -> EffectFn:
        marker = _EffectMarker(name=func.__name__, dependencies=dependencies_list)
        setattr(func, _EFFECT_MARKER_KEY, marker)
        return func

    return decorated


def _is_valid_effect_dependency(dependency: Any) -> TypeIs[_EffectDependency]:
    return isinstance(dependency, _StateMarker) or (
        isinstance(dependency, type) and issubclass(dependency, Hook)
    )


def _find_effect_markers(obj: object) -> list[_EffectMarker]:
    """Find instances of effect in the class.

    Args:
        obj: The object to search.

    Returns:
        Dictionary of effect name and effect marker.
    """
    ret_val: list[_EffectMarker] = []
    for name in dir(obj):
        value = getattr(obj, name)
        if marker := getattr(value, _EFFECT_MARKER_KEY, None):
            if marker.name != name:
                msg = "Effect name mismatch"
                raise RuntimeError(msg)
            ret_val.append(marker)
    return ret_val


_COMPONENT_STYLESHEET_RESET: StyleDict = {
    "background-color": "transparent",
    "border-radius": "0px",
    "border": "none",
    "margin": "0px",
    "padding": "0px",
}
"""Prevent Qt's unexpected behavior from inheriting parent's style."""


_LAST_ACCESSED_STATE_KEY = "__mui_last_accessed_attr__"


def _track_last_accessed_state(state: State[Any]) -> None:
    frame = inspect.currentframe()
    caller_frame = frame and frame.f_back
    caller_frame = caller_frame and caller_frame.f_back
    if caller_frame:
        caller_frame.f_locals[_LAST_ACCESSED_STATE_KEY] = state


def _pop_last_accessed_state(value: _T) -> State[_T] | None:
    # Check if we can bind to another object's state.
    # TODO: what if there are multiple intermediate stack frames?
    #   should it be global?
    frame = inspect.currentframe()
    caller_frame = frame and frame.f_back
    caller_frame = caller_frame and caller_frame.f_back
    state = (
        # Pop so it can't be rebound accidentally.
        caller_frame.f_locals.pop(_LAST_ACCESSED_STATE_KEY, None)
        if caller_frame
        else None
    )
    # Ensure it's the same value. This isn't always reliable but should
    # work for now...
    # TODO: additional checks? check id of values? types? code lineno? not itself?
    if state and isinstance(state, State) and state.get_value() is value:
        return cast("State[_T]", state)
    return None


class Component(QWidget, metaclass=_ComponentMeta):
    """Base class for all widgets."""

    sx = use_state(cast("StyleDict", {}))

    focused = use_state(False)
    """State version of Qt's `focus` property.

    This is only for reading the focus state and creating dependencies.
    To set focus, use Qt's built in focus handling functions for more
    flexibility.
    """

    _size = use_state(QSize())
    """Internal state for Qt `size` property."""

    _children = use_state([])
    """Internal state for Qt `children` property."""

    def __init__(self) -> None:
        super().__init__()

        self.__instantiate_state_variables()
        self.__bind_effects()

        # Make qt stylesheets work properly!
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, on=True)

        self.should_propagate_click = True
        """Whether the click event should be propagated to the parent widget."""

    def __instantiate_state_variables(self) -> None:
        """Create State instances from class variables."""
        for marker in _find_state_markers(self):
            state = State(marker.default_value, marker.name)
            state.setParent(self)
            setattr(self, marker.name, marker.default_value)
            # Propagate the internal state value to the mem var proxy.
            # This won't cause an infinite loop since the setter checks
            # if the value is different. It will eventually stabilize.
            # Use super so that it doesn't try to call the set_value of
            # the state again.
            state.changed.connect(partial(super().__setattr__, marker.name))
            # Apply transition if specified.
            if marker.transition:
                state.set_transition(
                    _TransitionConfig(marker.transition, marker.easing),
                )

    # Prevent IDEs from showing misspelled variables as valid.
    if not TYPE_CHECKING:

        def __getattribute__(self, name: str) -> Any:
            actual_value = super().__getattribute__(name)
            if name in {
                Component._find_state.__name__,
                Component.findChild.__name__,
            }:
                # Prevent recursion error. These are used below.
                return actual_value
            state = self._find_state(name)
            if state:
                # A state variable was accessed. Track it for binding.
                _track_last_accessed_state(state)
            return actual_value

        def __setattr__(self, name: str, value: Any) -> None:
            if state := self._find_state(name):
                if other_state := _pop_last_accessed_state(value):
                    state.bind(other_state)
                elif state.transition:
                    state.animate_to(
                        value,
                        state.transition.duration_ms,
                        state.transition.easing,
                    )
                else:
                    state.set_value(value)
            super().__setattr__(name, value)

    def _find_state(self, name: str) -> State[Any] | None:
        """Find state variable by name."""
        return self.findChild(
            cast("type[State[Any]]", State),
            name,
            Qt.FindChildOption.FindDirectChildrenOnly,
        )

    @overload
    def set_state(self, name: str) -> Callable[[Any], None]: ...

    @overload
    def set_state(self, name: str, value: Any) -> None: ...

    def set_state(
        self,
        name: str,
        value: Any = undefined,
    ) -> Callable[[Any], None] | None:
        """Set a state variable by name.

        Args:
            name: Name of the state variable.
            value: Value to set if needed.

        Returns:
            If no value is specified, a function is returned that can be
            used to set the value later.
            If a value is specified, None is returned.

        Raises:
            AttributeError: If no state with the given name exists.
        """
        state = self._find_state(name)
        if state is None:
            raise AttributeError
        if value is undefined:
            return state.set_value
        state.set_value(value)
        return None

    def __bind_effects(self) -> None:
        """Bind effects to the newly created variables."""
        for effect_marker in _find_effect_markers(self):
            # Get the function object from the class.
            func = getattr(self, effect_marker.name)
            for dependency in effect_marker.dependencies:
                if isinstance(dependency, _StateMarker):
                    # Find the corresponding variable object.
                    state = self._find_state(dependency.name)
                    if not isinstance(state, State):
                        msg = f"Invalid dependency for {dependency.name}: '{state}'"
                        raise TypeError(msg)
                    state.changed.connect(func)
                else:
                    # Dependency is a hook, so connect to its on_change signal.
                    hook_instance = dependency.get()
                    hook_instance.on_change.connect(func)
            # Call the function to apply the initial state. The timer
            # ensures the derived class's constructor is finished first.
            QTimer.singleShot(0, func)

    def overlay_widget(
        self,
        widget: QWidget,
        margins: QMargins | None = None,
        *,
        center: bool = False,
    ) -> None:
        """Overlay a widget on top of this widget.

        Ownership will also be set to this widget.
        """
        widget.setParent(self)
        if self.layout() is not None:
            msg = "Multiple overlay widgets not supported yet"
            raise NotImplementedError(msg)
        # TODO: add some other kind of 'overlay' layout similar to graphics anchors?
        layout = QVBoxLayout(self)
        layout.setSpacing(0)
        layout.setContentsMargins(margins or QMargins())
        if center:
            layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        layout.addWidget(widget)

    def set_transition(
        self,
        state: Any,
        duration_ms: int,
        easing: QEasingCurve.Type = QEasingCurve.Type.Linear,
    ) -> None:
        """Set a transition for a state variable.

        Args:
            state: Target state variable.
            duration_ms: Animation duration, in milliseconds.
            easing: Easing curve to use for the animation.
        """
        if state_obj := _pop_last_accessed_state(state):
            state_obj.set_transition(_TransitionConfig(duration_ms, easing))

    @effect(sx, ThemeHook)
    def _apply_sx(self) -> None:
        """Apply the sx property to the widget."""
        sx = {**_COMPONENT_STYLESHEET_RESET, **self.sx}
        # Special handling for opacity - pass to a graphics effect.
        if (opacity := sx.pop("opacity", None)) is not None:
            if not isinstance(opacity, (float, DesignToken)):
                raise TypeError
            self._apply_opacity_from_sx(opacity)
        qss = convert_sx_to_qss(sx)
        self.setStyleSheet(qss)

    def _apply_opacity_from_sx(self, opacity: float | DesignToken) -> None:
        if effect := self.graphicsEffect():
            # Reuse existing effect unless one is already set - Qt can
            # only have one.
            if not isinstance(effect, QGraphicsOpacityEffect):
                raise RuntimeError
        else:
            # Create a new effect.
            effect = QGraphicsOpacityEffect(self)
            self.setGraphicsEffect(effect)
        resolved_opacity = resolve_token_or_value(opacity)
        if not isinstance(resolved_opacity, float):
            raise TypeError
        effect.setOpacity(resolved_opacity)

    def resizeEvent(self, event: QResizeEvent) -> None:  # noqa: N802
        super().resizeEvent(event)
        self._size = self.size()

    @effect(QWidget.size)
    def _apply_size(self) -> None:
        """Apply the size property to the widget."""
        self.resize(self.size())

    def focusInEvent(self, event: QFocusEvent) -> None:  # noqa: N802
        self.focused = True
        return super().focusInEvent(event)

    def focusOutEvent(self, event: QFocusEvent) -> None:  # noqa: N802
        self.focused = False
        return super().focusOutEvent(event)

    def setFocusProxy(self, w: QWidget | None) -> None:  # noqa: N802
        # Intercept the focus proxy to listen to focus events correctly,
        # since Qt won't propagate the focus In/Out events to this
        # widget.
        if w:
            # TODO: edge cases, remove filter from previous focus proxy, unit tests
            w.installEventFilter(self)

        # Wrong Qt type annotation - should be QWidget | None.
        return super().setFocusProxy(w)  # type: ignore[arg-type]

    def eventFilter(self, watched: QObject, event: QEvent) -> bool:  # noqa: N802
        if watched is self.focusProxy():
            # Intercept the focus events from the focus proxy.
            if event.type() == QEvent.Type.FocusIn:
                self.focusInEvent(cast("QFocusEvent", event))
                return False  # Focus proxy should handle it too.
            if event.type() == QEvent.Type.FocusOut:
                self.focusOutEvent(cast("QFocusEvent", event))
                return False  # Focus proxy should handle it too.
        return super().eventFilter(watched, event)

    hovered = use_state(False)
    pressed = use_state(False)
    clicked: Signal

    def mousePressEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        if event.button() == Qt.MouseButton.LeftButton:
            self.pressed = True
        return super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        if event.button() == Qt.MouseButton.LeftButton:
            self.pressed = False
            mouse_inside = self.rect().contains(event.pos())
            if mouse_inside:
                self.clicked.emit()
                if not self.should_propagate_click:
                    return None
        return super().mouseReleaseEvent(event)

    def enterEvent(self, event: QEnterEvent) -> None:  # noqa: N802
        self.hovered = True
        return super().enterEvent(event)

    def leaveEvent(self, event: QEvent) -> None:  # noqa: N802
        self.hovered = False
        return super().leaveEvent(event)

    def childEvent(self, event: QChildEvent) -> None:  # noqa: N802
        self._children = self.children()
        super().childEvent(event)



"""Design tokens for md.sys.color."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

error = _define_token("md.ref.palette.error40")
error_container = _define_token("md.ref.palette.error90")
inverse_on_surface = _define_token("md.ref.palette.neutral95")
inverse_primary = _define_token("md.ref.palette.primary80")
inverse_surface = _define_token("md.ref.palette.neutral20")
on_error = _define_token("md.ref.palette.error100")
on_error_container = _define_token("md.ref.palette.error30")
on_primary = _define_token("md.ref.palette.primary100")
on_primary_container = _define_token("md.ref.palette.primary30")
on_secondary = _define_token("md.ref.palette.secondary100")
on_secondary_container = _define_token("md.ref.palette.secondary30")
on_surface = _define_token("md.ref.palette.neutral10")
on_surface_variant = _define_token("md.ref.palette.neutral-variant30")
on_tertiary = _define_token("md.ref.palette.tertiary30")
on_tertiary_container = _define_token("md.ref.palette.tertiary30")
outline = _define_token("md.ref.palette.neutral-variant50")
outline_variant = _define_token("md.ref.palette.neutral-variant80")
primary = _define_token("md.ref.palette.primary40")
primary_container = _define_token("md.ref.palette.primary90")
secondary = _define_token("md.ref.palette.secondary40")
secondary_container = _define_token("md.ref.palette.secondary90")
shadow = _define_token("md.ref.palette.neutral0")
surface = _define_token("md.ref.palette.neutral98")
surface_bright = _define_token("md.ref.palette.neutral98")
surface_container = _define_token("md.ref.palette.neutral94")
surface_container_high = _define_token("md.ref.palette.neutral92")
surface_container_highest = _define_token("md.ref.palette.neutral90")
surface_container_low = _define_token("md.ref.palette.neutral96")
surface_container_lowest = _define_token("md.ref.palette.neutral100")
surface_dim = _define_token("md.ref.palette.neutral87")
surface_tint = _define_token("md.sys.color.primary")
surface_variant = _define_token("md.ref.palette.neutral-variant90")
tertiary = _define_token("md.ref.palette.tertiary40")
tertiary_container = _define_token("md.ref.palette.tertiary90")
background = _define_token("md.ref.palette.white")



"""Design Token system.

Hardcoded values are available for all tokens. They can also be
overridden with an exported theme from Material Theme Builder plugin for
Figma, or dynamically generated.
"""

from material_ui.tokens._utils import (
    DesignToken,  # noqa: F401
    resolve_token,  # noqa: F401
    override_token,  # noqa: F401
)



"""Design tokens for md.comp.snackbar."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

action_focus_label_text_color = _define_token("md.sys.color.inverse-primary")
action_focus_state_layer_color = _define_token("md.sys.color.inverse-primary")
action_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
action_hover_label_text_color = _define_token("md.sys.color.inverse-primary")
action_hover_state_layer_color = _define_token("md.sys.color.inverse-primary")
action_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
action_label_text_color = _define_token("md.sys.color.inverse-primary")
action_label_text_font = _define_token("md.sys.typescale.label-large.font")
action_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
action_label_text_size = _define_token("md.sys.typescale.label-large.size")
action_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
action_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
action_pressed_label_text_color = _define_token("md.sys.color.inverse-primary")
action_pressed_state_layer_color = _define_token("md.sys.color.inverse-primary")
action_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
container_color = _define_token("md.sys.color.inverse-surface")
container_elevation = _define_token("md.sys.elevation.level3")
container_shadow_color = _define_token("md.sys.color.shadow")
container_shape = _define_token("md.sys.shape.corner.extra-small")
icon_color = _define_token("md.sys.color.inverse-on-surface")
icon_focus_icon_color = _define_token("md.sys.color.inverse-on-surface")
icon_focus_state_layer_color = _define_token("md.sys.color.inverse-on-surface")
icon_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
icon_hover_icon_color = _define_token("md.sys.color.inverse-on-surface")
icon_hover_state_layer_color = _define_token("md.sys.color.inverse-on-surface")
icon_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
icon_pressed_icon_color = _define_token("md.sys.color.inverse-on-surface")
icon_pressed_state_layer_color = _define_token("md.sys.color.inverse-on-surface")
icon_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
icon_size = _define_token(24)
supporting_text_color = _define_token("md.sys.color.inverse-on-surface")
supporting_text_font = _define_token("md.sys.typescale.body-medium.font")
supporting_text_line_height = _define_token("md.sys.typescale.body-medium.line-height")
supporting_text_size = _define_token("md.sys.typescale.body-medium.size")
supporting_text_tracking = _define_token("md.sys.typescale.body-medium.tracking")
supporting_text_weight = _define_token("md.sys.typescale.body-medium.weight")
with_single_line_container_height = _define_token(48)
with_two_lines_container_height = _define_token(68)



from pytest_mock import MockerFixture
from pytestqt.qtbot import QtBot
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QSizePolicy

from material_ui.text_fields._base_text_field import BaseTextField


def test_BaseTextField_value_set_from_typing(qtbot: QtBot):
    text_field = BaseTextField()
    qtbot.addWidget(text_field)
    # QTest doesn't seem to support focus, so type the text directly on
    # the line edit.
    qtbot.keyClicks(text_field._line_edit, "Hello")
    assert text_field.value == "Hello"


def test_BaseTextField_click_to_focus(qtbot: QtBot, mocker: MockerFixture):
    text_field = BaseTextField()
    qtbot.addWidget(text_field)
    # Assume the focus is propagated correctly if setFocus is called.
    spy = mocker.spy(text_field, "setFocus")
    qtbot.mouseClick(text_field, Qt.MouseButton.LeftButton)
    assert spy.called


def test_BaseTextField_size_hint(qtbot: QtBot):
    text_field = BaseTextField()
    text_field.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
    qtbot.addWidget(text_field)
    size_hint = text_field.sizeHint()
    assert 190 < size_hint.width() < 210
    assert 40 < size_hint.height() < 70



"""Design tokens for md.ref.palette."""

from qtpy.QtGui import QColor

from material_ui.tokens._utils import define_token as _define_token

black = _define_token(QColor("#000000"))
error0 = _define_token(QColor("#000000"))
error10 = _define_token(QColor("#410e0b"))
error100 = _define_token(QColor("#ffffff"))
error20 = _define_token(QColor("#601410"))
error30 = _define_token(QColor("#8c1d18"))
error40 = _define_token(QColor("#b3261e"))
error50 = _define_token(QColor("#dc362e"))
error60 = _define_token(QColor("#e46962"))
error70 = _define_token(QColor("#ec928e"))
error80 = _define_token(QColor("#f2b8b5"))
error90 = _define_token(QColor("#f9dedc"))
error95 = _define_token(QColor("#fceeee"))
error99 = _define_token(QColor("#fffbf9"))
neutral_variant0 = _define_token(QColor("#000000"))
neutral_variant10 = _define_token(QColor("#1d1a22"))
neutral_variant100 = _define_token(QColor("#ffffff"))
neutral_variant20 = _define_token(QColor("#322f37"))
neutral_variant30 = _define_token(QColor("#49454f"))
neutral_variant40 = _define_token(QColor("#605d66"))
neutral_variant50 = _define_token(QColor("#79747e"))
neutral_variant60 = _define_token(QColor("#938f99"))
neutral_variant70 = _define_token(QColor("#aea9b4"))
neutral_variant80 = _define_token(QColor("#cac4d0"))
neutral_variant90 = _define_token(QColor("#e7e0ec"))
neutral_variant95 = _define_token(QColor("#f5eefa"))
neutral_variant99 = _define_token(QColor("#fffbfe"))
neutral0 = _define_token(QColor("#000000"))
neutral10 = _define_token(QColor("#1d1b20"))
neutral100 = _define_token(QColor("#ffffff"))
neutral12 = _define_token(QColor("#211f26"))
neutral17 = _define_token(QColor("#2b2930"))
neutral20 = _define_token(QColor("#322f35"))
neutral22 = _define_token(QColor("#36343b"))
neutral24 = _define_token(QColor("#3b383e"))
neutral30 = _define_token(QColor("#48464c"))
neutral4 = _define_token(QColor("#0f0d13"))
neutral40 = _define_token(QColor("#605d64"))
neutral50 = _define_token(QColor("#79767d"))
neutral6 = _define_token(QColor("#141218"))
neutral60 = _define_token(QColor("#938f96"))
neutral70 = _define_token(QColor("#aea9b1"))
neutral80 = _define_token(QColor("#cac5cd"))
neutral87 = _define_token(QColor("#ded8e1"))
neutral90 = _define_token(QColor("#e6e0e9"))
neutral92 = _define_token(QColor("#ece6f0"))
neutral94 = _define_token(QColor("#f3edf7"))
neutral95 = _define_token(QColor("#f5eff7"))
neutral96 = _define_token(QColor("#f7f2fa"))
neutral98 = _define_token(QColor("#fef7ff"))
neutral99 = _define_token(QColor("#fffbff"))
primary0 = _define_token(QColor("#000000"))
primary10 = _define_token(QColor("#21005d"))
primary100 = _define_token(QColor("#ffffff"))
primary20 = _define_token(QColor("#381e72"))
primary30 = _define_token(QColor("#4f378b"))
primary40 = _define_token(QColor("#6750a4"))
primary50 = _define_token(QColor("#7f67be"))
primary60 = _define_token(QColor("#9a82db"))
primary70 = _define_token(QColor("#b69df8"))
primary80 = _define_token(QColor("#d0bcff"))
primary90 = _define_token(QColor("#eaddff"))
primary95 = _define_token(QColor("#f6edff"))
primary99 = _define_token(QColor("#fffbfe"))
secondary0 = _define_token(QColor("#000000"))
secondary10 = _define_token(QColor("#1d192b"))
secondary100 = _define_token(QColor("#ffffff"))
secondary20 = _define_token(QColor("#332d41"))
secondary30 = _define_token(QColor("#4a4458"))
secondary40 = _define_token(QColor("#625b71"))
secondary50 = _define_token(QColor("#7a7289"))
secondary60 = _define_token(QColor("#958da5"))
secondary70 = _define_token(QColor("#b0a7c0"))
secondary80 = _define_token(QColor("#ccc2dc"))
secondary90 = _define_token(QColor("#e8def8"))
secondary95 = _define_token(QColor("#f6edff"))
secondary99 = _define_token(QColor("#fffbfe"))
tertiary0 = _define_token(QColor("#000000"))
tertiary10 = _define_token(QColor("#31111d"))
tertiary100 = _define_token(QColor("#ffffff"))
tertiary20 = _define_token(QColor("#492532"))
tertiary30 = _define_token(QColor("#633b48"))
tertiary40 = _define_token(QColor("#7d5260"))
tertiary50 = _define_token(QColor("#986977"))
tertiary60 = _define_token(QColor("#b58392"))
tertiary70 = _define_token(QColor("#d29dac"))
tertiary80 = _define_token(QColor("#efb8c8"))
tertiary90 = _define_token(QColor("#ffd8e4"))
tertiary95 = _define_token(QColor("#ffecf1"))
tertiary99 = _define_token(QColor("#fffbfa"))
white = _define_token(QColor("#ffffff"))



"""Text Fields."""

from material_ui.text_fields.filled_text_field import FilledTextField
from material_ui.text_fields.outlined_text_field import OutlinedTextField

__all__ = [
    "FilledTextField",
    "OutlinedTextField",
]



from typing import TYPE_CHECKING, Literal, cast

from qtpy.QtCore import Qt
from qtpy.QtGui import QFont
from qtpy.QtWidgets import QLabel

from material_ui._component import Component, effect, use_state
from material_ui._font_utils import install_default_fonts
from material_ui.theming.theme_hook import ThemeHook
from material_ui.tokens import md_sys_color, md_sys_typescale
from material_ui.tokens._utils import DesignToken, resolve_token, resolve_token_or_value

if TYPE_CHECKING:
    from qtpy.QtGui import QColor

TypographyVariant = Literal[
    "display-large",
    "display-medium",
    "headline-large",
    "headline-medium",
    "headline-small",
    "title-large",
    "title-medium",
    "title-small",
    "body-large",
    "body-medium",
    "body-small",
    "label-large",
    "label-large-prominent",
    "label-medium",
    "label-medium-prominent",
    "label-small",
]

_VARIANT_SETTINGS_MAPPING: dict[
    TypographyVariant,
    tuple[DesignToken, DesignToken, DesignToken],
] = {
    "display-large": (
        md_sys_typescale.display_large_font,
        md_sys_typescale.display_large_size,
        md_sys_typescale.display_large_weight,
    ),
    "display-medium": (
        md_sys_typescale.display_medium_font,
        md_sys_typescale.display_medium_size,
        md_sys_typescale.display_medium_weight,
    ),
    "headline-large": (
        md_sys_typescale.headline_large_font,
        md_sys_typescale.headline_large_size,
        md_sys_typescale.headline_large_weight,
    ),
    "headline-medium": (
        md_sys_typescale.headline_medium_font,
        md_sys_typescale.headline_medium_size,
        md_sys_typescale.headline_medium_weight,
    ),
    "headline-small": (
        md_sys_typescale.headline_small_font,
        md_sys_typescale.headline_small_size,
        md_sys_typescale.headline_small_weight,
    ),
    "title-large": (
        md_sys_typescale.title_large_font,
        md_sys_typescale.title_large_size,
        md_sys_typescale.title_large_weight,
    ),
    "title-medium": (
        md_sys_typescale.title_medium_font,
        md_sys_typescale.title_medium_size,
        md_sys_typescale.title_medium_weight,
    ),
    "title-small": (
        md_sys_typescale.title_small_font,
        md_sys_typescale.title_small_size,
        md_sys_typescale.title_small_weight,
    ),
    "body-large": (
        md_sys_typescale.body_large_font,
        md_sys_typescale.body_large_size,
        md_sys_typescale.body_large_weight,
    ),
    "body-medium": (
        md_sys_typescale.body_medium_font,
        md_sys_typescale.body_medium_size,
        md_sys_typescale.body_medium_weight,
    ),
    "body-small": (
        md_sys_typescale.body_small_font,
        md_sys_typescale.body_small_size,
        md_sys_typescale.body_small_weight,
    ),
    "label-large": (
        md_sys_typescale.label_large_font,
        md_sys_typescale.label_large_size,
        md_sys_typescale.label_large_weight,
    ),
    "label-large-prominent": (
        md_sys_typescale.label_large_font,
        md_sys_typescale.label_large_size,
        md_sys_typescale.label_large_weight_prominent,
    ),
    "label-medium": (
        md_sys_typescale.label_medium_font,
        md_sys_typescale.label_medium_size,
        md_sys_typescale.label_medium_weight,
    ),
    "label-medium-prominent": (
        md_sys_typescale.label_medium_font,
        md_sys_typescale.label_medium_size,
        md_sys_typescale.label_medium_weight_prominent,
    ),
    "label-small": (
        md_sys_typescale.label_small_font,
        md_sys_typescale.label_small_size,
        md_sys_typescale.label_small_weight,
    ),
}


class Typography(Component):
    """Typography helps make writing legible and beautiful."""

    text = use_state("")
    """The writing to display."""

    color = use_state(cast("DesignToken | QColor", md_sys_color.on_surface))
    """Text color."""

    variant = use_state(cast("TypographyVariant | None", None))
    """Typography variant to control the font values from a preset."""

    font_family = use_state(md_sys_typescale.body_medium_font)
    """Font family."""

    font_size = use_state(md_sys_typescale.body_medium_size)
    """Font size defined by a design token. Units are in DP."""

    font_weight = use_state(md_sys_typescale.body_medium_weight)
    """Font weight defined by a design token."""

    alignment = use_state(cast("Qt.AlignmentFlag", Qt.AlignmentFlag()))  # type: ignore[call-arg]
    """Text alignment within the widget's geometry."""

    def __init__(self) -> None:
        super().__init__()

        install_default_fonts()

        self._label = QLabel()
        self.overlay_widget(self._label)

    @effect(alignment)
    def _apply_alignment(self) -> None:
        self._label.setAlignment(self.alignment)

    @effect(text)
    def _apply_text(self) -> None:
        self._label.setText(self.text)

    @effect(variant)
    def _apply_font_settings_from_variant(self) -> None:
        if not self.variant:
            return
        font_family, font_size, font_weight = _VARIANT_SETTINGS_MAPPING[self.variant]
        self.font_family = font_family
        self.font_size = font_size
        self.font_weight = font_weight

    @effect(font_family, font_size, font_weight, color, ThemeHook)
    def _apply_styles(self) -> None:
        self.sx = {**self.sx, "color": resolve_token_or_value(self.color)}

        # Can't control font-weight in qt stylesheets, so use QFont
        # instead. This also allows controlling variable axes.
        font = QFont(cast("str", resolve_token(self.font_family)))
        # Use pixel size instead of point size - point size makes it
        # look too big. DPI scaling seems to be correct like this too.
        font.setPixelSize(cast("int", resolve_token_or_value(self.font_size)))
        # Use variable axis (Qt>=6) instead of QFont.Weight enum.
        weight_value = cast("int", resolve_token_or_value(self.font_weight))
        font.setVariableAxis(QFont.Tag("wght"), weight_value)
        # Fix blurriness - see https://stackoverflow.com/a/61858057
        font.setHintingPreference(QFont.HintingPreference.PreferNoHinting)

        self._label.setFont(font)



"""Text Button component."""

from material_ui.buttons._button_base import ButtonBase
from material_ui.tokens import md_comp_text_button as tokens


class TextButton(ButtonBase):
    """TextButton."""

    def __init__(self) -> None:
        super().__init__()
        self._container.opacity = 0.0
        self._ripple.color = tokens.pressed_state_layer_color
        self._label.font_family = tokens.label_text_font
        self._label.font_size = tokens.label_text_size
        self._label.font_weight = tokens.label_text_weight
        self._label.color = tokens.label_text_color



"""Basic shape utility widget."""

from typing import cast

from qtpy.QtCore import Qt
from qtpy.QtGui import QColor

from material_ui._component import Component, effect, use_state
from material_ui.theming.theme_hook import ThemeHook
from material_ui.tokens import md_sys_color, md_sys_shape
from material_ui.tokens._utils import (
    DesignToken,
    find_root_token,
    resolve_token,
    resolve_token_or_value,
)

_OPACITY_MIN_THRESHOLD = 1.0e-2
"""Minimum opacity value for backgrounds.

If not checking this threshold, Qt will just go full opaque...
"""


class Shape(Component):
    """A blank component with common shape features."""

    visible = use_state(True)
    corner_shape = use_state(md_sys_shape.corner_none)
    color = use_state(cast("QColor | DesignToken", QColor("transparent")))
    opacity = use_state(cast("float | DesignToken", 1.0))

    @effect(corner_shape, Component.size)
    def _apply_corner_shape(self) -> None:
        token = find_root_token(self.corner_shape)
        # TODO: make the shape non identical tokens not compare equal
        if token is md_sys_shape.corner_none:
            update = {"border-radius": 0}
        elif token is md_sys_shape.corner_extra_extra_small:
            update = {"border-radius": 2}
        elif token is md_sys_shape.corner_extra_small:
            update = {"border-radius": 4}
        elif token is md_sys_shape.corner_extra_small_top:
            update = {"border-top-left-radius": 4, "border-top-right-radius": 4}
        elif token is md_sys_shape.corner_small:
            update = {"border-radius": 8}
        elif token is md_sys_shape.corner_medium:
            update = {"border-radius": 12}
        elif token is md_sys_shape.corner_large:
            update = {"border-radius": 16}
        elif token is md_sys_shape.corner_large_top:
            update = {"border-top-left-radius": 16, "border-top-right-radius": 16}
        elif token is md_sys_shape.corner_large_start:
            update = {"border-top-left-radius": 16, "border-bottom-left-radius": 16}
        elif token is md_sys_shape.corner_large_end:
            update = {"border-top-right-radius": 16, "border-bottom-right-radius": 16}
        elif token is md_sys_shape.corner_extra_large:
            update = {"border-radius": 28}
        elif token is md_sys_shape.corner_extra_large_top:
            update = {"border-top-left-radius": 28, "border-top-right-radius": 28}
        elif token is md_sys_shape.corner_full:
            half_size = min(self.width(), self.height()) // 2
            update = {"border-radius": half_size}
        else:
            raise ValueError
        self.sx = {**self.sx, **update}

    @effect(visible)
    def _apply_visible(self) -> None:
        self.setVisible(self.visible)

    @effect(color, opacity, ThemeHook)
    def _apply_background_color(self) -> None:
        color = resolve_token_or_value(self.color)
        opacity = resolve_token_or_value(self.opacity)
        if opacity < _OPACITY_MIN_THRESHOLD:
            opacity = 0.0
        # Make a copy to keep the design token's value unmodified.
        color = QColor(color)
        color.setAlphaF(opacity)
        self.sx = {**self.sx, "background-color": color}


class Line(Component):
    """A straight line."""

    color = use_state(md_sys_color.outline)
    thickness = use_state(cast("int | DesignToken", 1))
    orientation = use_state(Qt.Orientation.Horizontal)

    @effect(color, thickness, orientation)
    def _apply_line(self) -> None:
        self.sx = {"background-color": self.color}

        # Set one of the dimensions to the thickness. Parent component
        # will have to set the other dimension.
        match self.orientation:
            case Qt.Orientation.Horizontal:
                self.setFixedHeight(self.resolved_thickness())
            case Qt.Orientation.Vertical:
                self.setFixedWidth(self.resolved_thickness())
            case _:
                raise ValueError

    def resolved_thickness(self) -> int:
        """Return the resolved thickness of the line."""
        if isinstance(self.thickness, DesignToken):
            if isinstance(resolved := resolve_token(self.thickness), int):
                return resolved
            raise TypeError
        return self.thickness



"""Design tokens for md.comp.assist-chip."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_height = _define_token(32)
container_shape = _define_token("md.sys.shape.corner.small")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
disabled_label_text_color = _define_token("md.sys.color.on-surface")
disabled_label_text_opacity = _define_token(0.38)
dragged_container_elevation = _define_token("md.sys.elevation.level4")
dragged_label_text_color = _define_token("md.sys.color.on-surface")
dragged_state_layer_color = _define_token("md.sys.color.on-surface")
dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
elevated_container_color = _define_token("md.sys.color.surface-container-low")
elevated_container_elevation = _define_token("md.sys.elevation.level1")
elevated_container_shadow_color = _define_token("md.sys.color.shadow")
elevated_disabled_container_color = _define_token("md.sys.color.on-surface")
elevated_disabled_container_elevation = _define_token("md.sys.elevation.level0")
elevated_disabled_container_opacity = _define_token(0.12)
elevated_focus_container_elevation = _define_token("md.sys.elevation.level1")
elevated_hover_container_elevation = _define_token("md.sys.elevation.level2")
elevated_pressed_container_elevation = _define_token("md.sys.elevation.level1")
flat_container_elevation = _define_token("md.sys.elevation.level0")
flat_disabled_outline_color = _define_token("md.sys.color.on-surface")
flat_disabled_outline_opacity = _define_token(0.12)
flat_focus_outline_color = _define_token("md.sys.color.on-surface")
flat_outline_color = _define_token("md.sys.color.outline-variant")
flat_outline_width = _define_token(1)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_label_text_color = _define_token("md.sys.color.on-surface")
focus_state_layer_color = _define_token("md.sys.color.on-surface")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_label_text_color = _define_token("md.sys.color.on-surface")
hover_state_layer_color = _define_token("md.sys.color.on-surface")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
label_text_color = _define_token("md.sys.color.on-surface")
label_text_font = _define_token("md.sys.typescale.label-large.font")
label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
label_text_size = _define_token("md.sys.typescale.label-large.size")
label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
label_text_weight = _define_token("md.sys.typescale.label-large.weight")
pressed_label_text_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
with_icon_disabled_icon_color = _define_token("md.sys.color.on-surface")
with_icon_disabled_icon_opacity = _define_token(0.38)
with_icon_dragged_icon_color = _define_token("md.sys.color.primary")
with_icon_focus_icon_color = _define_token("md.sys.color.primary")
with_icon_hover_icon_color = _define_token("md.sys.color.primary")
with_icon_icon_color = _define_token("md.sys.color.primary")
with_icon_icon_size = _define_token(18)
with_icon_pressed_icon_color = _define_token("md.sys.color.primary")



"""Design tokens for md.comp.bottom-app-bar."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.surface-container")
container_elevation = _define_token("md.sys.elevation.level2")
container_height = _define_token(80)
container_shape = _define_token("md.sys.shape.corner.none")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
with_fab_container_height = _define_token(72)



"""Sample of using the icons."""

from material_ui import Component
from material_ui._component import use_state
from material_ui.icon import Icon
from material_ui.layout_basics import Row, Stack
from material_ui.switch import Switch
from material_ui.tokens import md_sys_color
from material_ui.typography import Typography
from qtpy.QtCore import QMargins, Qt
from qtpy.QtWidgets import QApplication, QGridLayout

ICONS = ["star", "arrow_drop_down", "more_vert", "check", "close", "add"]


class IconsSample(Component):
    filled = use_state(False)
    icon_style = use_state("outlined")

    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        main_row = Row()

        # TODO: make a grid widget / flex box
        icon_grid = Component()
        icon_grid_layout = QGridLayout(icon_grid)
        self._icons: list[Icon] = []
        for i, icon_name in enumerate(ICONS):
            icon = Icon()
            icon.icon_name = icon_name
            icon.filled = self.filled
            icon_grid_layout.addWidget(icon, i // 3, i % 3)
            self._icons.append(icon)

        main_row.add_widget(icon_grid)

        filters_box = Stack()
        filters_box.margins = QMargins(10, 10, 10, 10)
        filters_box.sx = {"background-color": md_sys_color.surface_container}

        filled_row = Row()
        filled_row.gap = 5

        filled_switch = Switch()
        filled_switch.selected = self.filled
        filled_switch.on_change.connect(self.set_state("filled"))
        filled_row.add_widget(filled_switch)

        filled_label = Typography()
        filled_label.variant = "body-large"
        filled_label.alignment = Qt.AlignmentFlag.AlignVCenter
        filled_label.text = "Filled"
        filled_row.add_widget(filled_label)

        filters_box.add_widget(filled_row)

        main_row.add_widget(filters_box)

        self.overlay_widget(main_row)


def main() -> None:
    """Main function."""
    app = QApplication()
    window = IconsSample()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Design tokens for md.comp.filled-tonal-icon-button."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.secondary-container")
container_height = _define_token(40)
container_shape = _define_token("md.sys.shape.corner.full")
container_size = _define_token(40)
container_width = _define_token(40)
disabled_container_color = _define_token("md.sys.color.on-surface")
disabled_container_opacity = _define_token(0.12)
disabled_icon_color = _define_token("md.sys.color.on-surface")
disabled_icon_opacity = _define_token(0.38)
focus_icon_color = _define_token("md.sys.color.on-secondary-container")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_icon_color = _define_token("md.sys.color.on-secondary-container")
hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
icon_color = _define_token("md.sys.color.on-secondary-container")
icon_size = _define_token(24)
pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
selected_container_color = _define_token("md.sys.color.secondary-container")
toggle_selected_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_icon_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
toggle_selected_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
toggle_unselected_focus_icon_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_hover_icon_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_icon_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_pressed_icon_color = _define_token("md.sys.color.on-surface-variant")
toggle_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
unselected_container_color = _define_token("md.sys.color.surface-container-highest")



"""Components to simplify layout of a few items."""

from typing import cast

from qtpy.QtCore import QMargins, Qt
from qtpy.QtWidgets import QHBoxLayout, QVBoxLayout, QWidget

from material_ui._component import Component, effect, use_state

_DefaultAlignment = cast("Qt.AlignmentFlag", Qt.AlignmentFlag())  # type: ignore[assignment, call-arg]


class Row(Component):
    """A horizontal container."""

    alignment = use_state(_DefaultAlignment)
    gap = use_state(0)
    margins = use_state(QMargins())

    def __init__(self) -> None:
        super().__init__()
        self._hbox = QHBoxLayout(self)

    def add_widget(self, widget: QWidget) -> None:
        """Add a widget to the row."""
        self._hbox.addWidget(widget)

    @effect(gap, alignment, margins)
    def _update_hbox(self) -> None:
        self._hbox.setSpacing(self.gap)
        self._hbox.setAlignment(self.alignment)
        self._hbox.setContentsMargins(self.margins)


class Stack(Component):
    """A vertical container."""

    alignment = use_state(_DefaultAlignment)
    gap = use_state(0)
    margins = use_state(QMargins())

    def __init__(
        self,
        *,
        alignment: Qt.AlignmentFlag | None = None,
        gap: int | None = None,
        margins: QMargins | None = None,
    ) -> None:
        super().__init__()

        self._vbox = QVBoxLayout(self)

        if alignment is not None:
            self.alignment = alignment
        if gap is not None:
            self.gap = gap
        if margins is not None:
            self.margins = margins

    def add_widget(self, widget: QWidget) -> None:
        """Add a widget to the stack."""
        self._vbox.addWidget(widget)

    @effect(gap, alignment, margins)
    def _update_vbox(self) -> None:
        self._vbox.setSpacing(self.gap)
        self._vbox.setAlignment(self.alignment)
        self._vbox.setContentsMargins(self.margins)



"""Design tokens for md.comp.secondary-navigation-tab."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.primary")
active_indicator_height = _define_token(2)
active_label_text_color = _define_token("md.sys.color.on-surface")
container_color = _define_token("md.sys.color.surface")
container_elevation = _define_token("md.sys.elevation.level0")
container_height = _define_token(48)
container_shadow_color = _define_token("md.sys.color.shadow")
container_shape = _define_token("md.sys.shape.corner.none")
divider_color = _define_token("md.sys.color.surface-variant")
divider_height = _define_token(1)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.inner-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_label_text_color = _define_token("md.sys.color.on-surface")
focus_state_layer_color = _define_token("md.sys.color.on-surface")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_label_text_color = _define_token("md.sys.color.on-surface")
hover_state_layer_color = _define_token("md.sys.color.on-surface")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
inactive_label_text_color = _define_token("md.sys.color.on-surface-variant")
label_text_font = _define_token("md.sys.typescale.title-small.font")
label_text_line_height = _define_token("md.sys.typescale.title-small.line-height")
label_text_size = _define_token("md.sys.typescale.title-small.size")
label_text_tracking = _define_token("md.sys.typescale.title-small.tracking")
label_text_weight = _define_token("md.sys.typescale.title-small.weight")
pressed_label_text_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
with_icon_active_icon_color = _define_token("md.sys.color.on-surface")
with_icon_focus_icon_color = _define_token("md.sys.color.on-surface")
with_icon_hover_icon_color = _define_token("md.sys.color.on-surface")
with_icon_icon_size = _define_token(24)
with_icon_inactive_icon_color = _define_token("md.sys.color.on-surface-variant")
with_icon_pressed_icon_color = _define_token("md.sys.color.on-surface")



"""Progress indicators."""

from material_ui.progress_indicators.circular_progress import CircularProgress
from material_ui.progress_indicators.linear_progress import LinearProgress

__all__ = [
    "CircularProgress",
    "LinearProgress",
]



"""Design tokens for md.comp.extended-fab."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

branded_container_color = _define_token("md.sys.color.surface-container-high")
branded_container_elevation = _define_token("md.sys.elevation.level3")
branded_container_height = _define_token(56)
branded_container_shadow_color = _define_token("md.sys.color.shadow")
branded_container_shape = _define_token("md.sys.shape.corner.large")
branded_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
branded_focus_container_elevation = _define_token("md.sys.elevation.level3")
branded_focus_indicator_color = _define_token("md.sys.color.secondary")
branded_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
branded_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
branded_focus_label_text_color = _define_token("md.sys.color.primary")
branded_focus_state_layer_color = _define_token("md.sys.color.primary")
branded_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
branded_hover_container_elevation = _define_token("md.sys.elevation.level4")
branded_hover_label_text_color = _define_token("md.sys.color.primary")
branded_hover_state_layer_color = _define_token("md.sys.color.primary")
branded_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
branded_icon_size = _define_token(36)
branded_label_text_color = _define_token("md.sys.color.on-surface")
branded_label_text_font = _define_token("md.sys.typescale.label-large.font")
branded_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
branded_label_text_size = _define_token("md.sys.typescale.label-large.size")
branded_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
branded_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
branded_lowered_container_color = _define_token("md.sys.color.surface-container-low")
branded_lowered_container_elevation = _define_token("md.sys.elevation.level1")
branded_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
branded_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
branded_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
branded_pressed_container_elevation = _define_token("md.sys.elevation.level3")
branded_pressed_label_text_color = _define_token("md.sys.color.primary")
branded_pressed_state_layer_color = _define_token("md.sys.color.primary")
branded_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
primary_container_color = _define_token("md.sys.color.primary-container")
primary_container_elevation = _define_token("md.sys.elevation.level3")
primary_container_height = _define_token(56)
primary_container_shadow_color = _define_token("md.sys.color.shadow")
primary_container_shape = _define_token("md.sys.shape.corner.large")
primary_focus_container_elevation = _define_token("md.sys.elevation.level3")
primary_focus_icon_color = _define_token("md.sys.color.on-primary-container")
primary_focus_indicator_color = _define_token("md.sys.color.secondary")
primary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
primary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
primary_focus_label_text_color = _define_token("md.sys.color.on-primary-container")
primary_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
primary_hover_container_elevation = _define_token("md.sys.elevation.level4")
primary_hover_icon_color = _define_token("md.sys.color.on-primary-container")
primary_hover_label_text_color = _define_token("md.sys.color.on-primary-container")
primary_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
primary_icon_color = _define_token("md.sys.color.on-primary-container")
primary_icon_size = _define_token(24)
primary_label_text_color = _define_token("md.sys.color.on-primary-container")
primary_label_text_font = _define_token("md.sys.typescale.label-large.font")
primary_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
primary_label_text_size = _define_token("md.sys.typescale.label-large.size")
primary_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
primary_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
primary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
primary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
primary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
primary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
primary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
primary_pressed_icon_color = _define_token("md.sys.color.on-primary-container")
primary_pressed_label_text_color = _define_token("md.sys.color.on-primary-container")
primary_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
secondary_container_color = _define_token("md.sys.color.secondary-container")
secondary_container_elevation = _define_token("md.sys.elevation.level3")
secondary_container_height = _define_token(56)
secondary_container_shadow_color = _define_token("md.sys.color.shadow")
secondary_container_shape = _define_token("md.sys.shape.corner.large")
secondary_focus_container_elevation = _define_token("md.sys.elevation.level3")
secondary_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_focus_indicator_color = _define_token("md.sys.color.secondary")
secondary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
secondary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
secondary_focus_label_text_color = _define_token("md.sys.color.on-secondary-container")
secondary_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
secondary_hover_container_elevation = _define_token("md.sys.elevation.level4")
secondary_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_hover_label_text_color = _define_token("md.sys.color.on-secondary-container")
secondary_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
secondary_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_icon_size = _define_token(24)
secondary_label_text_color = _define_token("md.sys.color.on-secondary-container")
secondary_label_text_font = _define_token("md.sys.typescale.label-large.font")
secondary_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
secondary_label_text_size = _define_token("md.sys.typescale.label-large.size")
secondary_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
secondary_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
secondary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
secondary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
secondary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
secondary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
secondary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
secondary_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_pressed_label_text_color = _define_token("md.sys.color.on-secondary-container")
secondary_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
surface_container_color = _define_token("md.sys.color.surface-container-high")
surface_container_elevation = _define_token("md.sys.elevation.level3")
surface_container_height = _define_token(56)
surface_container_shadow_color = _define_token("md.sys.color.shadow")
surface_container_shape = _define_token("md.sys.shape.corner.large")
surface_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
surface_focus_container_elevation = _define_token("md.sys.elevation.level3")
surface_focus_icon_color = _define_token("md.sys.color.primary")
surface_focus_indicator_color = _define_token("md.sys.color.secondary")
surface_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
surface_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
surface_focus_label_text_color = _define_token("md.sys.color.primary")
surface_focus_state_layer_color = _define_token("md.sys.color.primary")
surface_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
surface_hover_container_elevation = _define_token("md.sys.elevation.level4")
surface_hover_icon_color = _define_token("md.sys.color.primary")
surface_hover_label_text_color = _define_token("md.sys.color.primary")
surface_hover_state_layer_color = _define_token("md.sys.color.primary")
surface_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
surface_icon_color = _define_token("md.sys.color.primary")
surface_icon_size = _define_token(24)
surface_label_text_color = _define_token("md.sys.color.primary")
surface_label_text_font = _define_token("md.sys.typescale.label-large.font")
surface_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
surface_label_text_size = _define_token("md.sys.typescale.label-large.size")
surface_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
surface_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
surface_lowered_container_color = _define_token("md.sys.color.surface-container-low")
surface_lowered_container_elevation = _define_token("md.sys.elevation.level1")
surface_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
surface_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
surface_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
surface_pressed_container_elevation = _define_token("md.sys.elevation.level3")
surface_pressed_icon_color = _define_token("md.sys.color.primary")
surface_pressed_label_text_color = _define_token("md.sys.color.primary")
surface_pressed_state_layer_color = _define_token("md.sys.color.primary")
surface_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
tertiary_container_color = _define_token("md.sys.color.tertiary-container")
tertiary_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_container_height = _define_token(56)
tertiary_container_shadow_color = _define_token("md.sys.color.shadow")
tertiary_container_shape = _define_token("md.sys.shape.corner.large")
tertiary_focus_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_focus_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_focus_indicator_color = _define_token("md.sys.color.secondary")
tertiary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
tertiary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
tertiary_focus_label_text_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
tertiary_hover_container_elevation = _define_token("md.sys.elevation.level4")
tertiary_hover_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_hover_label_text_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
tertiary_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_icon_size = _define_token(24)
tertiary_label_text_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_label_text_font = _define_token("md.sys.typescale.label-large.font")
tertiary_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
tertiary_label_text_size = _define_token("md.sys.typescale.label-large.size")
tertiary_label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
tertiary_label_text_weight = _define_token("md.sys.typescale.label-large.weight")
tertiary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
tertiary_lowered_presed_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_pressed_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_pressed_label_text_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")



"""Design tokens for md.comp.progress-indicator."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_track_space = _define_token(4)
active_indicator_color = _define_token("md.sys.color.primary")
active_indicator_shape = _define_token("md.sys.shape.corner.full")
active_indicator_thickness = _define_token(4)
stop_indicator_color = _define_token("md.sys.color.primary")
stop_indicator_shape = _define_token(4)
stop_indicator_size = _define_token(4)
track_color = _define_token("md.sys.color.secondary-container")
track_shape = _define_token("md.sys.shape.corner.full")
track_thickness = _define_token(4)



"""Design tokens for md.comp.linear-progress-indicator."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.primary")
active_indicator_height = _define_token(4)
active_indicator_shape = _define_token("md.sys.shape.corner.none")
four_color_active_indicator_four_color = _define_token("md.sys.color.tertiary-container")
four_color_active_indicator_one_color = _define_token("md.sys.color.primary")
four_color_active_indicator_three_color = _define_token("md.sys.color.tertiary")
four_color_active_indicator_two_color = _define_token("md.sys.color.primary-container")
track_color = _define_token("md.sys.color.surface-container-highest")
track_height = _define_token(4)
track_shape = _define_token("md.sys.shape.corner.none")



"""A 'hidden' component that provides drop shadow."""

from qtpy.QtCore import QPointF, QRectF, Qt
from qtpy.QtGui import QColor, QPainter
from qtpy.QtWidgets import QGraphicsDropShadowEffect, QGraphicsEffect

from material_ui._component import Component, effect, use_state
from material_ui.shape import Shape
from material_ui.tokens import md_sys_color, md_sys_elevation, md_sys_shape
from material_ui.tokens._utils import DesignToken, find_root_token, resolve_token

# TODO: implement token generator overrides for missing tokens for level5

_ELEVATION_KEY_OFFSET_MAP: dict[DesignToken, QPointF] = {
    md_sys_elevation.level0: QPointF(0.0, 0.0),
    md_sys_elevation.level1: QPointF(0.0, 1.0),
    md_sys_elevation.level2: QPointF(0.0, 1.0),
    md_sys_elevation.level3: QPointF(0.0, 1.0),
    md_sys_elevation.level4: QPointF(0.0, 2.0),
    # md_sys_elevation.level5: QPointF(0.0, 4.0),
}

_ELEVATION_KEY_BLUR_RADIUS_MAP: dict[DesignToken, int] = {
    md_sys_elevation.level0: 0,
    md_sys_elevation.level1: 2,
    md_sys_elevation.level2: 2,
    md_sys_elevation.level3: 3,
    md_sys_elevation.level4: 3,
    # md_sys_elevation.level5: 4,
}

_ELEVATION_AMBIENT_OFFSET_MAP: dict[DesignToken, QPointF] = {
    md_sys_elevation.level0: QPointF(0.0, 0.0),
    md_sys_elevation.level1: QPointF(0.0, 1.0),
    md_sys_elevation.level2: QPointF(0.0, 2.0),
    md_sys_elevation.level3: QPointF(0.0, 3.0),
    md_sys_elevation.level4: QPointF(0.0, 6.0),
    # md_sys_elevation.level5: QPointF(0.0, 8.0),
}

_ELEVATION_AMBIENT_BLUR_RADIUS_MAP: dict[DesignToken, int] = {
    md_sys_elevation.level0: 0,
    md_sys_elevation.level1: 3,
    md_sys_elevation.level2: 6,
    md_sys_elevation.level3: 8,
    md_sys_elevation.level4: 10,
    # md_sys_elevation.level5: 12,
}


class Elevation(Component):
    """Elevation (aka drop shadow)."""

    elevation = use_state(md_sys_elevation.level0)
    shadow_color = use_state(md_sys_color.shadow)
    corner_shape = use_state(md_sys_shape.corner_none)

    def __init__(self) -> None:
        super().__init__()

        # Have to set a background for Qt's DropShadow to work...!
        # TODO: remove once using custom graphics effect
        self.sx.set({"background-color": "white"})

        # TODO: also add key shadow
        self._ambient_shadow = QGraphicsDropShadowEffect()
        self._ambient_shadow.setParent(self)
        self.setGraphicsEffect(self._ambient_shadow)

    @effect(corner_shape, Component.size)
    def _apply_corner_shape(self):
        """Apply corner shape."""
        # TODO: move to common place?
        shape = find_root_token(self.corner_shape)
        if shape is md_sys_shape.corner_none:
            new_radius = 0
        elif shape is md_sys_shape.corner_full:
            new_radius = min(self.width(), self.height()) // 2
        else:
            raise RuntimeError("unsupported corner shape", shape)
        self.sx.set(lambda prev: prev | {"border-radius": new_radius})

    @effect(shadow_color)
    def _apply_shadow_colors(self):
        """Apply shadow colors."""
        color = resolve_token(self.shadow_color)
        if not isinstance(color, QColor):
            raise RuntimeError(
                f"invalid shadow_color token: expected QColor, got {type(color).__name__}"
            )

        ambient_color = QColor(color)
        # TODO: once using multi shadow, alpha values are: key=0.3, ambient=0.15
        ambient_color.setAlphaF(0.35)
        self._ambient_shadow.setColor(ambient_color)

    @effect(elevation)
    def _apply_elevation(self):
        """Apply elevation."""
        elevation = find_root_token(self.elevation)
        self._ambient_shadow.setBlurRadius(
            _ELEVATION_AMBIENT_BLUR_RADIUS_MAP[elevation]
        )
        self._ambient_shadow.setOffset(_ELEVATION_AMBIENT_OFFSET_MAP[elevation])



"""Design tokens for md.comp.slider."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_container_opacity = _define_token(1.0)
active_handle_height = _define_token(44)
active_handle_leading_space = _define_token(6)
active_handle_padding = _define_token(6)
active_handle_shape = _define_token("md.sys.shape.corner.full")
active_handle_trailing_space = _define_token(6)
active_handle_width = _define_token(4)
active_stop_indicator_container_color = _define_token("md.sys.color.on-primary")
active_stop_indicator_container_opacity = _define_token(1.0)
active_track_color = _define_token("md.sys.color.primary")
active_track_height = _define_token(16)
active_track_shape = _define_token("md.sys.shape.corner.full")
active_track_shape_leading = _define_token("md.sys.shape.corner.full")
active_track_shape_trailing = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
disabled_active_stop_indicator_container_color = _define_token("md.sys.color.inverse-on-surface")
disabled_active_track_color = _define_token("md.sys.color.on-surface")
disabled_active_track_opacity = _define_token(0.38)
disabled_handle_color = _define_token("md.sys.color.on-surface")
disabled_handle_elevation = _define_token("md.sys.elevation.level0")
disabled_handle_opacity = _define_token(0.38)
disabled_handle_width = _define_token(4)
disabled_inactive_stop_indicator_container_color = _define_token("md.sys.color.on-surface")
disabled_inactive_track_color = _define_token("md.sys.color.on-surface")
disabled_inactive_track_opacity = _define_token(0.12)
disabled_stop_indicator_color = _define_token("md.sys.color.on-surface")
disabled_stop_indicator_color_selected = _define_token("md.sys.color.inverse-on-surface")
disabled_stop_indicator_container_color = _define_token("md.sys.color.on-surface")
disabled_stop_indicator_container_opacity = _define_token(0.38)
focus_active_track_color = _define_token("md.sys.color.primary")
focus_handle_color = _define_token("md.sys.color.primary")
focus_handle_width = _define_token(2)
focus_inactive_track_color = _define_token("md.sys.color.secondary-container")
focus_state_layer_color = _define_token("md.sys.color.primary")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
focus_stop_color = _define_token("md.sys.color.primary")
handle_color = _define_token("md.sys.color.primary")
handle_elevation = _define_token("md.sys.elevation.level1")
handle_height = _define_token(44)
handle_shadow_color = _define_token("md.sys.color.shadow")
handle_shape = _define_token("md.sys.shape.corner.full")
handle_width = _define_token(4)
hover_handle_color = _define_token("md.sys.color.primary")
hover_handle_width = _define_token(4)
hover_state_layer_color = _define_token("md.sys.color.primary")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
hover_stop_color = _define_token("md.sys.color.primary")
inactive_container_opacity = _define_token(1.0)
inactive_stop_indicator_container_color = _define_token("md.sys.color.on-secondary-container")
inactive_stop_indicator_container_opacity = _define_token(1.0)
inactive_track_color = _define_token("md.sys.color.secondary-container")
inactive_track_height = _define_token(16)
inactive_track_shape = _define_token("md.sys.shape.corner.full")
label_container_color = _define_token("md.sys.color.primary")
label_container_elevation = _define_token("md.sys.elevation.level0")
label_container_height = _define_token(28)
label_label_text_color = _define_token("md.sys.color.inverse-on-surface")
label_label_text_font = _define_token("md.sys.typescale.label-medium.font")
label_label_text_font_size = _define_token("md.sys.typescale.label-medium.size")
label_label_text_line_height = _define_token("md.sys.typescale.label-medium.line-height")
label_label_text_size = _define_token("md.sys.typescale.label-medium.size")
label_label_text_tracking = _define_token("md.sys.typescale.label-medium.tracking")
label_label_text_weight = _define_token("md.sys.typescale.label-medium.weight")
pressed_active_track_color = _define_token("md.sys.color.primary")
pressed_handle_color = _define_token("md.sys.color.primary")
pressed_handle_width = _define_token(2)
pressed_inactive_track_color = _define_token("md.sys.color.secondary-container")
pressed_state_layer_color = _define_token("md.sys.color.primary")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
pressed_stop_color = _define_token("md.sys.color.primary")
slider_active_handle_color = _define_token("md.sys.color.primary")
state_layer_size = _define_token(40)
stop_indicator_color = _define_token("md.sys.color.on-secondary-container")
stop_indicator_color_selected = _define_token("md.sys.color.on-primary")
stop_indicator_shape = _define_token("md.sys.shape.corner.full")
stop_indicator_size = _define_token(4)
stop_indicator_trailing_space = _define_token(6)
track_elevation = _define_token("md.sys.elevation.level0")
value_indicator_active_bottom_space = _define_token(12)
value_indicator_container_color = _define_token("md.sys.color.inverse-surface")
value_indicator_label_label_text_color = _define_token("md.sys.color.inverse-on-surface")
value_indicator_label_label_text_font = _define_token("md.sys.typescale.label-large.font")
value_indicator_label_label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
value_indicator_label_label_text_size = _define_token("md.sys.typescale.label-large.size")
value_indicator_label_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
value_indicator_label_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
with_overlap_handle_outline_color = _define_token("md.sys.color.on-primary")
with_overlap_handle_outline_width = _define_token(1)
with_tick_marks_active_container_color = _define_token("md.sys.color.on-primary")
with_tick_marks_active_container_opacity = _define_token(0.38)
with_tick_marks_container_shape = _define_token("md.sys.shape.corner.full")
with_tick_marks_container_size = _define_token(2)
with_tick_marks_disabled_container_color = _define_token("md.sys.color.on-surface")
with_tick_marks_disabled_container_opacity = _define_token(0.38)
with_tick_marks_inactive_container_color = _define_token("md.sys.color.on-surface-variant")
with_tick_marks_inactive_container_opacity = _define_token(0.38)



"""Dynamic color integration with materialyoucolor library."""

from functools import partial

from materialyoucolor.dynamiccolor.dynamic_color import DynamicColor
from materialyoucolor.dynamiccolor.material_dynamic_colors import MaterialDynamicColors
from materialyoucolor.scheme.dynamic_scheme import DynamicScheme
from qtpy.QtGui import QColor

from material_ui.tokens import md_sys_color
from material_ui.tokens._utils import DesignToken, override_token


def apply_dynamic_color_scheme(scheme: DynamicScheme) -> None:
    """Apply the dynamic color scheme to the application."""
    fn = partial(_apply_scheme_to_token, source_scheme=scheme)

    fn(md_sys_color.error, MaterialDynamicColors.error)
    fn(md_sys_color.error_container, MaterialDynamicColors.errorContainer)
    fn(md_sys_color.inverse_on_surface, MaterialDynamicColors.inverseOnSurface)
    fn(md_sys_color.inverse_primary, MaterialDynamicColors.inversePrimary)
    fn(md_sys_color.inverse_surface, MaterialDynamicColors.inverseSurface)
    fn(md_sys_color.on_error, MaterialDynamicColors.onError)
    fn(md_sys_color.on_error_container, MaterialDynamicColors.onErrorContainer)
    fn(md_sys_color.on_primary, MaterialDynamicColors.onPrimary)
    fn(md_sys_color.on_primary_container, MaterialDynamicColors.onPrimaryContainer)
    fn(md_sys_color.on_secondary, MaterialDynamicColors.onSecondary)
    fn(md_sys_color.on_secondary_container, MaterialDynamicColors.onSecondaryContainer)
    fn(md_sys_color.on_surface, MaterialDynamicColors.onSurface)
    fn(md_sys_color.on_surface_variant, MaterialDynamicColors.onSurfaceVariant)
    fn(md_sys_color.on_tertiary, MaterialDynamicColors.onTertiary)
    fn(md_sys_color.on_tertiary_container, MaterialDynamicColors.onTertiaryContainer)
    fn(md_sys_color.outline, MaterialDynamicColors.outline)
    fn(md_sys_color.outline_variant, MaterialDynamicColors.outlineVariant)
    fn(md_sys_color.primary, MaterialDynamicColors.primary)
    fn(md_sys_color.primary_container, MaterialDynamicColors.primaryContainer)
    fn(md_sys_color.secondary, MaterialDynamicColors.secondary)
    fn(md_sys_color.secondary_container, MaterialDynamicColors.secondaryContainer)
    fn(md_sys_color.shadow, MaterialDynamicColors.shadow)
    fn(md_sys_color.surface, MaterialDynamicColors.surface)
    fn(md_sys_color.surface_bright, MaterialDynamicColors.surfaceBright)
    fn(md_sys_color.surface_dim, MaterialDynamicColors.surfaceDim)
    fn(md_sys_color.surface_container, MaterialDynamicColors.surfaceContainer)
    fn(md_sys_color.surface_container_high, MaterialDynamicColors.surfaceContainerHigh)
    fn(
        md_sys_color.surface_container_highest,
        MaterialDynamicColors.surfaceContainerHighest,
    )
    fn(md_sys_color.surface_container_low, MaterialDynamicColors.surfaceContainerLow)
    fn(
        md_sys_color.surface_container_lowest,
        MaterialDynamicColors.surfaceContainerLowest,
    )
    fn(md_sys_color.surface_tint, MaterialDynamicColors.surfaceTint)
    fn(md_sys_color.surface_variant, MaterialDynamicColors.surfaceVariant)
    fn(md_sys_color.tertiary, MaterialDynamicColors.tertiary)
    fn(md_sys_color.tertiary_container, MaterialDynamicColors.tertiaryContainer)
    fn(md_sys_color.background, MaterialDynamicColors.background)


def _apply_scheme_to_token(
    target_token: DesignToken,
    source_color: DynamicColor,
    source_scheme: DynamicScheme,
) -> None:
    """Apply the dynamic color scheme to a specific token."""
    argb = source_color.get_argb(source_scheme)
    qt_color = QColor(argb & 0x00FFFFFF)
    override_token(target_token, qt_color)



"""Design tokens for md.comp.search-view."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.surface-container-high")
container_elevation = _define_token("md.sys.elevation.level3")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
divider_color = _define_token("md.sys.color.outline")
docked_container_shape = _define_token("md.sys.shape.corner.extra-large")
docked_header_container_height = _define_token(56)
full_screen_container_shape = _define_token("md.sys.shape.corner.none")
full_screen_header_container_height = _define_token(72)
header_input_text_color = _define_token("md.sys.color.on-surface")
header_input_text_font = _define_token("md.sys.typescale.body-large.font")
header_input_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
header_input_text_size = _define_token("md.sys.typescale.body-large.size")
header_input_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
header_input_text_weight = _define_token("md.sys.typescale.body-large.weight")
header_leading_icon_color = _define_token("md.sys.color.on-surface")
header_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
header_supporting_text_font = _define_token("md.sys.typescale.body-large.font")
header_supporting_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
header_supporting_text_size = _define_token("md.sys.typescale.body-large.size")
header_supporting_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
header_supporting_text_weight = _define_token("md.sys.typescale.body-large.weight")
header_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")



"""Design tokens for md.comp.switch."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

disabled_handle_elevation = _define_token("md.sys.elevation.level0")
disabled_handle_opacity = _define_token(0.38)
disabled_selected_handle_color = _define_token("md.sys.color.surface")
disabled_selected_handle_opacity = _define_token(1.0)
disabled_selected_icon_color = _define_token("md.sys.color.on-surface")
disabled_selected_icon_opacity = _define_token(0.38)
disabled_selected_track_color = _define_token("md.sys.color.on-surface")
disabled_track_opacity = _define_token(0.12)
disabled_unselected_handle_color = _define_token("md.sys.color.on-surface")
disabled_unselected_handle_opacity = _define_token(0.38)
disabled_unselected_icon_color = _define_token("md.sys.color.surface-container-highest")
disabled_unselected_icon_opacity = _define_token(0.38)
disabled_unselected_track_color = _define_token("md.sys.color.surface-container-highest")
disabled_unselected_track_outline_color = _define_token("md.sys.color.on-surface")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
handle_elevation = _define_token("md.sys.elevation.level1")
handle_height = _define_token(20)
handle_shadow_color = _define_token("md.sys.color.shadow")
handle_shape = _define_token("md.sys.shape.corner.full")
handle_width = _define_token(20)
pressed_handle_height = _define_token(28)
pressed_handle_width = _define_token(28)
selected_focus_handle_color = _define_token("md.sys.color.primary-container")
selected_focus_icon_color = _define_token("md.sys.color.on-primary-container")
selected_focus_state_layer_color = _define_token("md.sys.color.primary")
selected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
selected_focus_track_color = _define_token("md.sys.color.primary")
selected_handle_color = _define_token("md.sys.color.on-primary")
selected_handle_height = _define_token(24)
selected_handle_width = _define_token(24)
selected_hover_handle_color = _define_token("md.sys.color.primary-container")
selected_hover_icon_color = _define_token("md.sys.color.on-primary-container")
selected_hover_state_layer_color = _define_token("md.sys.color.primary")
selected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
selected_hover_track_color = _define_token("md.sys.color.primary")
selected_icon_color = _define_token("md.sys.color.on-primary-container")
selected_icon_size = _define_token(16)
selected_pressed_handle_color = _define_token("md.sys.color.primary-container")
selected_pressed_icon_color = _define_token("md.sys.color.on-primary-container")
selected_pressed_state_layer_color = _define_token("md.sys.color.primary")
selected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
selected_pressed_track_color = _define_token("md.sys.color.primary")
selected_track_color = _define_token("md.sys.color.primary")
state_layer_shape = _define_token("md.sys.shape.corner.full")
state_layer_size = _define_token(40)
track_height = _define_token(32)
track_outline_width = _define_token(2)
track_shape = _define_token("md.sys.shape.corner.full")
track_width = _define_token(52)
unselected_focus_handle_color = _define_token("md.sys.color.on-surface-variant")
unselected_focus_icon_color = _define_token("md.sys.color.surface-container-highest")
unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface")
unselected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
unselected_focus_track_color = _define_token("md.sys.color.surface-container-highest")
unselected_focus_track_outline_color = _define_token("md.sys.color.outline")
unselected_handle_color = _define_token("md.sys.color.outline")
unselected_handle_height = _define_token(16)
unselected_handle_width = _define_token(16)
unselected_hover_handle_color = _define_token("md.sys.color.on-surface-variant")
unselected_hover_icon_color = _define_token("md.sys.color.surface-container-highest")
unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface")
unselected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
unselected_hover_track_color = _define_token("md.sys.color.surface-container-highest")
unselected_hover_track_outline_color = _define_token("md.sys.color.outline")
unselected_icon_color = _define_token("md.sys.color.surface-container-highest")
unselected_icon_size = _define_token(16)
unselected_pressed_handle_color = _define_token("md.sys.color.on-surface-variant")
unselected_pressed_icon_color = _define_token("md.sys.color.surface-container-highest")
unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
unselected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
unselected_pressed_track_color = _define_token("md.sys.color.surface-container-highest")
unselected_pressed_track_outline_color = _define_token("md.sys.color.outline")
unselected_track_color = _define_token("md.sys.color.surface-container-highest")
unselected_track_outline_color = _define_token("md.sys.color.outline")
with_icon_handle_height = _define_token(24)
with_icon_handle_width = _define_token(24)



"""Design tokens for md.comp.filled-card."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.surface-container-highest")
container_elevation = _define_token("md.sys.elevation.level0")
container_shadow_color = _define_token("md.sys.color.shadow")
container_shape = _define_token("md.sys.shape.corner.medium")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
disabled_container_color = _define_token("md.sys.color.surface-variant")
disabled_container_elevation = _define_token("md.sys.elevation.level0")
disabled_container_opacity = _define_token(0.38)
dragged_container_elevation = _define_token("md.sys.elevation.level3")
dragged_state_layer_color = _define_token("md.sys.color.on-surface")
dragged_state_layer_opacity = _define_token("md.sys.state.dragged.state-layer-opacity")
focus_container_elevation = _define_token("md.sys.elevation.level0")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_state_layer_color = _define_token("md.sys.color.on-surface")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_container_elevation = _define_token("md.sys.elevation.level1")
hover_state_layer_color = _define_token("md.sys.color.on-surface")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
icon_color = _define_token("md.sys.color.primary")
icon_size = _define_token(24)
pressed_container_elevation = _define_token("md.sys.elevation.level0")
pressed_state_layer_color = _define_token("md.sys.color.on-surface")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")



"""Base class for button variants."""

from typing import cast

from qtpy.QtCore import QMargins, QSize, Qt
from qtpy.QtGui import QMouseEvent
from qtpy.QtWidgets import QHBoxLayout

from material_ui._component import Component, Signal, effect, use_state
from material_ui._lab import DropShadow
from material_ui.ripple import Ripple
from material_ui.shape import Shape

# Use the tokens from elevated_button for code completion, but they will
# get overridden by the subclasses.
from material_ui.tokens import md_comp_elevated_button as tokens
from material_ui.tokens._utils import resolve_token
from material_ui.typography import Typography

_TOUCH_AREA_Y_PADDING = 8
_TOUCH_AREA_MARGINS = QMargins(0, _TOUCH_AREA_Y_PADDING, 0, _TOUCH_AREA_Y_PADDING)


class ButtonBase(Component):
    """Base class for button variants."""

    clicked: Signal

    text = use_state("")

    hovered = use_state(False)
    pressed = use_state(False)

    def __init__(self) -> None:
        super().__init__()

        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.sx = {"margin": f"{_TOUCH_AREA_Y_PADDING}px 0px"}

        self._drop_shadow = DropShadow()
        self.setGraphicsEffect(self._drop_shadow)

        self._container = Shape()
        self._container.corner_shape = tokens.container_shape
        self._container.setParent(self)
        self._container.move(0, _TOUCH_AREA_Y_PADDING)

        self._state_layer = Shape()
        self._state_layer.setParent(self._container)
        self._state_layer.corner_shape = tokens.container_shape

        self._ripple = Ripple()
        self._ripple.setParent(self._container)

        container_layout = QHBoxLayout(self._container)
        container_layout.setContentsMargins(QMargins(24, 0, 24, 0))
        container_layout.setSpacing(0)

        self._label = Typography()
        self._label.text = self.text
        self._label.alignment = Qt.AlignmentFlag.AlignCenter
        container_layout.addWidget(self._label)

    def sizeHint(self) -> QSize:  # noqa: N802
        height = cast("int", resolve_token(tokens.container_height))
        return (
            self._container.sizeHint()
            # For some reason, setting the fixedHeight on the container
            # won't apply to its sizeHint, so set the height here.
            .expandedTo(QSize(0, height))
            .grownBy(_TOUCH_AREA_MARGINS)
        )

    @effect(Component.size)
    def _apply_element_sizes(self) -> None:
        # Since Qt has no 'overlay' layout, manually set these sizes.
        container_size = self.size().shrunkBy(_TOUCH_AREA_MARGINS)
        self._container.resize(container_size)
        self._state_layer.resize(container_size)
        self._ripple.resize(container_size)

    @effect(hovered)
    def _update_state_layer(self) -> None:
        self._state_layer.color = tokens.hover_state_layer_color
        self._state_layer.opacity = (
            tokens.hover_state_layer_opacity if self.hovered else 0.0
        )

    def mousePressEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        if event.button() == Qt.MouseButton.LeftButton:
            self._ripple.ripple_origin = event.position()
        return super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:  # noqa: N802
        if event.button() == Qt.MouseButton.LeftButton:
            self._ripple.ripple_origin = None
        return super().mouseReleaseEvent(event)



from pytestqt.qtbot import QtBot

from material_ui.checkbox import Checkbox


def test_Checkbox_basic_api(qtbot: QtBot):
    checkbox = Checkbox()
    checkbox.selected = True
    qtbot.addWidget(checkbox)



"""Sample of using the buttons."""

from functools import partial

from material_ui._component import Component
from material_ui.buttons import (
    ElevatedButton,
    FilledButton,
    FilledTonalButton,
    OutlinedButton,
    TextButton,
)
from material_ui.layout_basics import Row
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QApplication


class SampleButtons(Component):
    def __init__(self) -> None:
        super().__init__()
        self.resize(700, 200)

        row = Row()
        row.alignment = Qt.AlignmentFlag.AlignCenter
        row.gap = 30
        row.sx = {"background-color": "white"}
        self.overlay_widget(row)

        for variant, klass in {
            "Elevated": ElevatedButton,
            "Filled": FilledButton,
            "Tonal": FilledTonalButton,
            "Outlined": OutlinedButton,
            "Text": TextButton,
        }.items():
            button = klass()
            button.text = variant
            button.clicked.connect(partial(self._on_click_button, variant))
            row.add_widget(button)

    def _on_click_button(self, variant: str) -> None:
        print(f"Clicked: {variant}")


def main() -> None:
    app = QApplication()
    window = SampleButtons()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()
